/** This class extends SpellDictionaryHashMap and does the same as super class and something more.  
 * It is designed to use the spelling checker of multiple languages. In constructors it takes extra 
 * boolean parameter for language selection and instantiate the Transformator object accordingly! 
 * This is currently designed for Bengali and English!     
 *
 * @author Naushad UzZaman (naushadzaman@gmail.com)
 */
package com.swabunga.spell.engine;

/**
 * SpellDictionaryMultiLingual.java
 *
 *@author     Naushad UzZaman (naushadzaman@gmail.com)
 *@created    October 15, 2005
 *@version    0.1, December 10, 2005
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

public class SpellDictionaryMultiLingual extends SpellDictionaryHashMap{


    protected void initializeBanglaPhonetic(Reader phonetic) throws IOException{
        if (phonetic == null)
              tf = new BanglaMetaphone();
            else
              tf = new GenericTransformator(phonetic);
        
    }
    
    protected void initializeEnglishPhonetic(Reader phonetic) throws IOException{
        if (phonetic == null)
              tf = new DoubleMeta();
            else
              tf = new GenericTransformator(phonetic);
    }
    
    /**
     *  Constructor for the SpellDictionaryMultiLingual object
     *
     *@param  wordList                   Description of the Parameter
     *@param  phonetic                   Description of the Parameter
     *@exception  FileNotFoundException  Description of the Exception
     *@exception  IOException            Description of the Exception
     */
    public SpellDictionaryMultiLingual(File wordList, Reader phonetic, boolean language) throws FileNotFoundException, IOException {
        if(language==true){
            initializeBanglaPhonetic(phonetic);
        }
        else{
            initializeEnglishPhonetic(phonetic);
        }
        dictFile = wordList;
        createDictionary(new BufferedReader(new InputStreamReader(new FileInputStream(wordList), "UTF-8")));
    }
    
    
    /**
     *  Dictionary constructor that uses an aspell phonetic file to build the
     *  transformation table.
     *
     *@param  wordList         Description of the Parameter
     *@param  phonetic         Description of the Parameter
     *@exception  IOException  Description of the Exception
     */
    public SpellDictionaryMultiLingual(Reader wordList, Reader phonetic, boolean language) throws IOException {
        
        if(language==true){
            initializeBanglaPhonetic(phonetic);
        }
        else{
            initializeEnglishPhonetic(phonetic);
        }
        
        dictFile = null;
        createDictionary(new BufferedReader(wordList));
        
    }
}
