/*
 *  Copyright 1999-2002 Matthew Robinson and Pavel Vorobiev.
 *  All Rights Reserved.
 *
 *  ===================================================
 *  This program contains code from the book "Swing"
 *  2nd Edition by Matthew Robinson and Pavel Vorobiev
 *  http://www.spindoczine.com/sbe/
 *  ===================================================
 */

package net.sf.banglapad;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.DropTarget;
import java.awt.event.*;
import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.prefs.Preferences;

import javax.help.CSH;
import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.HTML;
import javax.swing.undo.*;
import javax.swing.text.Segment;

import java.net.URL;
/**
 * FIXME: Code Added by Zahurul
 */
import javax.swing.border.*;

import javax.swing.text.html.HTMLDocument;

import com.swabunga.spell.engine.SpellDictionary;
import com.swabunga.spell.engine.SpellDictionaryHashMap;

/**
 * 
 * BanglaPad
 * 
 * @author Zahurul Islam (zahurul@bracuniversity.net)
 * @author Naushad UzZaman (naushadzaman@gmail.com)
 * @created September 10, 2005
 * @version 0.1, December 10, 2005
 * @see http://sourceforge.net/projects/banglapad
 * 
 * Modified from VietPad
 * 
 * @author Quan Nguyen
 * @author Gero Herrmann
 * @created January 26, 2002
 * @version 1.6, 14 April 2005
 * @see http://vietpad.sourceforge.net/
 * 
 */
public class BanglaPad extends JFrame {

    final static boolean MAC_OS_X = System.getProperty("os.name").startsWith(
            "Mac");

    final static int TAB_SIZE = 4;

    final static Locale BANGLA = new Locale("bn", "BD");

    private final Clipboard clipboard = Toolkit.getDefaultToolkit()
            .getSystemClipboard();

    private final Rectangle screen = GraphicsEnvironment
            .getLocalGraphicsEnvironment().getMaximumWindowBounds();

    private final JPopupMenu popup = new JPopupMenu();

    private final UndoManager m_undo = new UndoManager();

    protected final int MENU_MASK = Toolkit.getDefaultToolkit()
            .getMenuShortcutKeyMask();

    protected final Preferences prefs = Preferences.userRoot().node(
            "/net/sourceforge/vietpad");

    protected final UndoableEditSupport undoSupport = new UndoableEditSupport();

    private boolean m_textChanged = true;

    private boolean toolBarOn;

    /**
     * FIXME: Code Extract or modification Should Extract or modify
     * viedModeon,smartMarkon,localVietOn
     */
    protected boolean smartMarkOn, banglaModeOn, defaultEOLOn, localeBanglaOn;

    private Action actionSave, actionCut, actionCopy, actionPaste,
            actionDelete, actionSelectAll;

    private Action m_redoAction, m_undoAction;

    protected HTMLDocument m_doc;

    private JMenuItem itemCut, itemCopy, itemDelete, itemPaste;

    private RawListener rawListener;

    protected javax.swing.filechooser.FileFilter utf16Filter, utf8Filter,
            bngFilter, txtFilter;

    protected ResourceBundle myResources;

    protected FindDialog m_findDialog;

    protected JButton banglaModeLabel;

    protected File m_currentFile;

    protected Font m_font;

    protected JFileChooser m_chooser;

    protected JMenuBar menuBar;

    protected JScrollPane scrollPane;

    protected JTextPane m_editor;

    protected JToolBar m_toolBar;

    protected static Locale systemDefault;

    private JFrame helptopicsFrame;

    protected MyHTMLEditorKit htmlKit;

    protected StyleContext m_context;

    protected int m_xStart = -1;

    protected int m_xFinish = -1;

    private static String fontName = " ";

    private static int fontSize = 0;

    protected boolean mSkipUpdate;

    protected JComboBox fontNameList;

    protected JComboBox fontSizeList;

    protected String[] fontFamily;

    protected String[] fontSizes;

    protected SmallToggleButton BoldButton;

    protected SmallToggleButton ItalicButton;

    protected SmallToggleButton underLineButton;

    private boolean charAttar = false;

	protected static String currentTmpFolder = null;/*currentTmpFolder is the systemTmpPath+TimeStamp*/

    protected boolean bngFormat = true;

    protected static String imagePath = null;

    protected String[] banglaFont;

    protected String[] englishFont;

    protected JMenuItem keybordMenu;

    /**
     * Constructor for the VietPad Object
     */
    public BanglaPad() {
        super(BPadLogger.APP_NAME);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        BPadLogger.logger.debug("Initializing BanglaPad constructor");

        imagePath = getClass().getResource("/org/gjt/sp/jedit/icons/")
                .toString();
        BPadLogger.logger.debug("ImagePath is loaded");

        if (MAC_OS_X && Locale.getDefault().getCountry().equals("BD")) {
            Locale.setDefault(BANGLA);
        }
        BPadLogger.logger.debug("Locale is set");
        systemDefault = Locale.getDefault();
        Locale
                .setDefault((localeBanglaOn = prefs.getBoolean("localeVN",
                        false))
                        || systemDefault.getLanguage().equals("bn") ? BANGLA
                        : Locale.US);
        myResources = ResourceBundle.getBundle("Resources");

        setSize(snap(prefs.getInt("frameWidth", 500), 300, screen.width), snap(
                prefs.getInt("frameHeight", 360), 150, screen.height));
        setLocation(snap(prefs
                .getInt("frameX", (screen.width - getWidth()) / 2), screen.x,
                screen.x + screen.width - getWidth()), snap(prefs.getInt(
                "frameY", screen.y + (screen.height - getHeight()) / 3),
                screen.y, screen.y + screen.height - getHeight()));
        m_font = new Font(prefs.get("fontName", MAC_OS_X ? "Lucida Grande"
                : "Mukti Narrow"), prefs.getInt("fontStyle", Font.PLAIN), prefs
                .getInt("fontSize", 12));
        toolBarOn = prefs.getBoolean("toolBar", true);
        banglaModeOn = prefs.getBoolean("vietMode", true);
        smartMarkOn = prefs.getBoolean("smartMark", true);
        defaultEOLOn = prefs.getBoolean("defaultEOL", true);
        BPadLogger.logger.debug("User preference is loaded");

        try {
            UIManager.setLookAndFeel(prefs.get("lookAndFeel", UIManager
                    .getSystemLookAndFeelClassName()));
        } catch (Exception e) {
            BPadLogger.logger
                    .debug("Problem  setLookAndFell() in BanglaPad class");
            // keep default LAF
        }

        rawListener = new RawListener();
        m_editor = new JTextPane();
        m_editor.setContentType("text/html");
        htmlKit = new MyHTMLEditorKit();
        m_context = new StyleContext();
        m_editor.setEditorKit(htmlKit);
        m_doc = (HTMLDocument) htmlKit.createDefaultDocument();
        m_editor.setStyledDocument(m_doc);

        CaretListener lst = new CaretListener() {
            public void caretUpdate(CaretEvent e) {
                showAttributes(e.getDot());
            }
        };
        m_editor.addCaretListener(lst);

        FocusListener flst = new FocusListener() {
            public void focusGained(FocusEvent e) {
                int len = m_editor.getDocument().getLength();
                if (m_xStart >= 0 && m_xFinish >= 0 && m_xStart < len
                        && m_xFinish < len)
                    if (m_editor.getCaretPosition() == m_xStart) {
                        m_editor.setCaretPosition(m_xFinish);
                        m_editor.moveCaretPosition(m_xStart);
                    } else
                        m_editor.select(m_xStart, m_xFinish);
            }

            public void focusLost(FocusEvent e) {
                m_xStart = m_editor.getSelectionStart();
                m_xFinish = m_editor.getSelectionEnd();
            }
        };
        m_editor.addFocusListener(flst);

        /**
         * Code Removed
         */
        /* m_doc = m_editor.getDocument(); */
        m_doc.addUndoableEditListener(rawListener);
        undoSupport.addUndoableEditListener(new SupportListener());
        /**
         * Code Removed
         */
        /* m_editor.setTabSize(TAB_SIZE); */
        // m_editor.setFont(m_font);
        /* m_editor.setWrapStyleWord(true); */
        m_editor.setMargin(new Insets(3, 2, 1, 1));
        m_editor.getInputMap().put(KeyStroke.getKeyStroke("ctrl H"), "none"); // Workaround
                                                                                // Bug
                                                                                // ID:
                                                                                // 6249912
                                                                                // &
                                                                                // 4782077

        m_editor.setDragEnabled(true);
        new DropTarget(m_editor, new TextDropTargetListener(this));

        scrollPane = new JScrollPane(m_editor);
        scrollPane
                .setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollPane.setAutoscrolls(true);
        getContentPane().add(scrollPane);

        /**
         * FIXME: Code Extract or Modification Here I only change the letter B
         * instead of V
         */
        banglaModeLabel = new JButton(banglaModeOn ? "B" : "E");
        banglaModeLabel.setFont(banglaModeLabel.getFont().deriveFont(
                Font.PLAIN, 24));
        banglaModeLabel.setForeground(Color.BLUE);
        banglaModeLabel.setContentAreaFilled(false);
        banglaModeLabel.setBorderPainted(false);
        banglaModeLabel.setFocusable(false);
        banglaModeLabel.setToolTipText(myResources.getString("Keyboard_Mode"));
        m_editor.addKeyListener(new BanglaKeyListener(m_editor));

        // Add BanglaKeyListener to JTextArea

        BPadLogger.logger.debug("Before calling createMenuBar() in VietPad");
        setJMenuBar(menuBar = createMenuBar());
        BPadLogger.logger.debug("After calling createMenubar() in VietPad");
        m_toolBar.setVisible(toolBarOn);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                quit();
            }

            public void windowActivated(WindowEvent e) {
                m_editor.requestFocus();
                updatePaste();
            }

            public void windowOpened(WindowEvent e) {
                setExtendedState(prefs.getInt("windowState", Frame.NORMAL));
            }
        });

        m_editor.addCaretListener(new CaretListener() {
            public void caretUpdate(CaretEvent e) {
                updateCutCopyDelete(e.getDot() != e.getMark());
            }
        });
        m_chooser = new JFileChooser();
        utf8Filter = new SimpleFilter("txt", "Unicode (UTF-8)");
        utf16Filter = new SimpleFilter("txt", "Unicode (UTF-16)");
        bngFilter = new SimpleFilter("bng", "BanglaPad Documents (.bng)");
        txtFilter = new SimpleFilter("txt", "Text Documents (.txt)");

        m_chooser.setAcceptAllFileFilterUsed(false);
        m_chooser.addChoosableFileFilter(bngFilter);
        m_chooser.addChoosableFileFilter(txtFilter);
        m_chooser.addChoosableFileFilter(utf8Filter);
        m_chooser.addChoosableFileFilter(utf16Filter);
        m_chooser.setFileFilter(m_chooser.getChoosableFileFilters()[prefs
                .getInt("filterIndex" + 5, 0)]);

        /**
         * Putting action for SPACE, it will put &nbsp instead of SPACE
         */

        m_editor.getInputMap().put(
                KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0), "check");
        m_editor.getActionMap().put("check", new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                int offset;
                offset = m_editor.getCaretPosition();
                try {
                    /**
                     * This insertHTML change fonts to monospaced, and it also
                     * uses <b>, however the forum claims that this <b> doesn't
                     * have any effect. But problem was it chages the font. so
                     * tried another one and this work well..
                     */
                    // ((HTMLEditorKit)htmlKit).insertHTML(m_doc, offset,
                    // "<b>\240</b>", 0, 0, HTML.Tag.B);
                    m_doc.insertString(offset, "\240", m_editor
                            .getInputAttributes());

                } catch (Exception ae) {
                    BPadLogger.logger.debug("Exception : " + ae.toString()
                            + " occured in VietPad");
                }
            }
        });

        m_chooser.setCurrentDirectory(new File(prefs.get("currentDirectory",
                System.getProperty("user.home"))));
        BPadLogger.logger.debug("Before calling newDocument() in VietPad");
        newDocument();
        BPadLogger.logger.debug("After calling newDocument() in VietPad");

    }

    /**
     * Create an icon from a graphics resource
     * 
     * @param imageName
     *            Name of icon
     * @return the icon created
     */
    public ImageIcon createIcon(String imageName) {
        return new ImageIcon(getClass().getResource(
                "/org/gjt/sp/jedit/icons/" + imageName + ".png"));
    }

    protected void showAttributes(int p) {
        BPadLogger.logger
                .debug("Beginging of showAttribute() in VietPad class eith param: "
                        + p);
        mSkipUpdate = true;
        AttributeSet attr = null;
        if (charAttar) {
            attr = m_doc.getCharacterElement(p).getAttributes();
            charAttar = false;
        } else {
            attr = htmlKit.getInputAttributes();
        }
        String name = StyleConstants.getFontFamily(attr);
        if (!fontName.equals(name)) {
            fontName = name;
            fontNameList.setSelectedItem(fontName);
        }
        int size = StyleConstants.getFontSize(attr);
        if (fontSize != size) {
            fontSize = size;
            fontSizeList.setSelectedItem(Integer.toString(fontSize));
        }
        boolean bold = StyleConstants.isBold(attr);
        if (bold != BoldButton.isSelected())
            BoldButton.setSelected(bold);
        boolean italic = StyleConstants.isItalic(attr);
        if (italic != ItalicButton.isSelected())
            ItalicButton.setSelected(italic);
        boolean underLine = StyleConstants.isUnderline(attr);
        if (underLine != underLineButton.isSelected())
            underLineButton.setSelected(underLine);
        mSkipUpdate = false;
        BPadLogger.logger.debug("End of showAttribute() in VietPad class");
    }

    protected void setAttributeSet(AttributeSet attr) {
        setAttributeSet(attr, false);
    }

    protected void setAttributeSet(AttributeSet attr,
            boolean setParagraphAttributes) {
        BPadLogger.logger.debug("Beginging of setAttribute() in VietPad class");
        if (mSkipUpdate)
            return;
        int xStart = m_editor.getSelectionStart();
        int xFinish = m_editor.getSelectionEnd();
        if (!m_editor.hasFocus()) {
            xStart = m_xStart;
            xFinish = m_xFinish;
        }
        if (setParagraphAttributes) {
            m_doc.setParagraphAttributes(xStart, xFinish - xStart, attr, false); // paragraphElement

        } else if (xStart != xFinish) {
            m_doc.setCharacterAttributes(xStart, xFinish - xStart, attr, false);
            charAttar = true;
        } else {
            MutableAttributeSet inputAttributes = htmlKit.getInputAttributes();
            inputAttributes.addAttributes(attr);
        }
        BPadLogger.logger.debug("End of setAttribute() in VietPad class");
    }

    /**
     * Create Menu bar
     * 
     * @return the MenuBar Object
     */
    private JMenuBar createMenuBar() {
        final JMenuBar menuBar = new JMenuBar();
        JMenu menu;
        JMenu subMenu;
        JMenuItem item;

        // File menu
        BPadLogger.logger.debug("Creating menubar in VietPad");
        menu = new JMenu(myResources.getString("File"));
        menu.setMnemonic('f');
        System.out.println(" menu font == " + menu.getFont().toString());
        Action actionNew = new AbstractAction(myResources.getString("New")) {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click New menu");
                if (promptToSave()) {
                    BPadLogger.logger
                            .debug("previous document saved and calling newDocument() as New's action");
                    newDocument();
                    BPadLogger.logger
                            .debug("new Document is added in textPane");
                }
            }
        };

        item = new JMenuItem(actionNew);
        item.setMnemonic('n');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, MENU_MASK));
        menu.add(item);

        Action actionOpen = new AbstractAction(myResources.getString("Open")
                + "...") {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click Open menu");
                if (promptToSave()) {
                    BPadLogger.logger
                            .debug("previous document saved and calling openDocument() as Open's action");
                    openDocument(null);
                    BPadLogger.logger
                            .debug("Doncument is opened and added in textPane");
                }
            }
        };

        item = new JMenuItem(actionOpen);
        item.setMnemonic('o');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, MENU_MASK));
        menu.add(item);

        actionSave = new AbstractAction(myResources.getString("Save")) {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click Save menu");
                BPadLogger.logger
                        .debug("Calling saveFile(false) as saveAction");
                saveFile(false);
                BPadLogger.logger.debug("File is saved as Save action");
            }
        };

        item = new JMenuItem(actionSave);
        item.setMnemonic('s');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, MENU_MASK));
        menu.add(item);

        Action actionSaveAs = new AbstractAction(myResources
                .getString("Save_As")
                + "...") {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User Click SaveAs menu");
                BPadLogger.logger
                        .debug("Calling saveFile(true) as saveAsAction");
                saveFile(true);
                BPadLogger.logger.debug("File is saved as SaveAs action");
            }
        };

        item = new JMenuItem(actionSaveAs);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, MENU_MASK
                | InputEvent.SHIFT_MASK));
        item.setMnemonic('a');
        menu.add(item);

        menu.addSeparator();

        Action actionExport = new AbstractAction("Export as HTML") {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click Export as HTML menu");
                BPadLogger.logger.debug("Calling exportHTML() in VietPad");
                exportHTML();
                BPadLogger.logger
                        .debug("current Doncument is saved as HTML cause of Export as HTML");
            }
        };

        item = new JMenuItem(actionExport);
        item.setMnemonic('e');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, MENU_MASK));
        menu.add(item);

        Action actionPageSetup = new AbstractAction(myResources
                .getString("Page_Setup")
                + "...") {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click Page Setup menu");
                pageSetup();
            }
        };

        item = menu.add(actionPageSetup);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, MENU_MASK
                + InputEvent.SHIFT_MASK));

        Action actionPrint = new AbstractAction(myResources.getString("Print")
                + "...") {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click Print menu");
                printPage();
            }
        };

        item = menu.add(actionPrint);
        item.setMnemonic('p');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, MENU_MASK));

        if (!MAC_OS_X) {
            menu.addSeparator();
            Action actionExit = new AbstractAction(myResources
                    .getString("Exit")) {
                public void actionPerformed(ActionEvent e) {
                    BPadLogger.logger.debug("User Click Exit menu");
                    quit();
                }
            };

            item = menu.add(actionExit);
            item.setMnemonic('x');
        }

        menuBar.add(menu);
        BPadLogger.logger.debug("Adding File toolbar utility");
        m_toolBar = new JToolBar();
        m_toolBar.add(new SmallButton(actionNew, myResources.getString("New"),
                createIcon("New")));
        m_toolBar.add(new SmallButton(actionOpen,
                myResources.getString("Open"), createIcon("Open")));
        m_toolBar.add(new SmallButton(actionSave,
                myResources.getString("Save"), createIcon("Save")));
        m_toolBar.add(new SmallButton(actionPrint, myResources
                .getString("Print"), createIcon("Print")));
        m_toolBar.addSeparator();

        // Edit menu

        menu = new JMenu(myResources.getString("Edit"));
        menu.setMnemonic('e');

        m_undoAction = new AbstractAction(myResources.getString("Undo")) {
            public void actionPerformed(ActionEvent e) {
                try {
                    BPadLogger.logger.debug("User Click Undo menu");
                    m_undo.undo();
                    updateSave(true);
                } catch (CannotUndoException ex) {
                    BPadLogger.logger.debug(myResources
                            .getString("Unable_to_undo:_")
                            + ex);
                }
                updateUndoRedo();
            }
        };

        item = menu.add(m_undoAction);
        item.setMnemonic('u');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, MENU_MASK));
        popup.add(m_undoAction);

        m_redoAction = new AbstractAction(myResources.getString("Redo")) {
            public void actionPerformed(ActionEvent e) {
                try {
                    BPadLogger.logger.debug("User Click Redo menu");
                    m_undo.redo();
                    updateSave(true);
                } catch (CannotRedoException ex) {
                    BPadLogger.logger.debug(myResources
                            .getString("Unable_to_redo:_")
                            + ex);
                }
                updateUndoRedo();
            }
        };

        item = menu.add(m_redoAction);
        item.setMnemonic('r');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, MENU_MASK));
        popup.add(m_redoAction);
        popup.addSeparator();

        menu.addSeparator();

        actionCut = new AbstractAction(myResources.getString("Cut")) {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User Click Cut menu");
                m_editor.cut();
                updatePaste();
            }
        };

        itemCut = menu.add(actionCut);
        itemCut.setMnemonic('t');
        itemCut
                .setAccelerator(KeyStroke
                        .getKeyStroke(KeyEvent.VK_X, MENU_MASK));
        popup.add(actionCut);

        actionCopy = new AbstractAction(myResources.getString("Copy")) {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User  click Copy menu");
                m_editor.copy();
                updatePaste();
            }
        };

        itemCopy = menu.add(actionCopy);
        itemCopy.setMnemonic('c');
        itemCopy.setAccelerator(KeyStroke
                .getKeyStroke(KeyEvent.VK_C, MENU_MASK));
        popup.add(actionCopy);

        actionPaste = new AbstractAction(myResources.getString("Paste")) {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User Click Paste menu");
                undoSupport.beginUpdate();
                m_editor.paste();
                undoSupport.endUpdate();
            }
        };

        itemPaste = menu.add(actionPaste);
        itemPaste.setMnemonic('p');
        itemPaste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
                MENU_MASK));
        popup.add(actionPaste);

        actionDelete = new AbstractAction(myResources.getString("Delete")) {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User Click Delete menu");
                m_editor.replaceSelection(null);
            }
        };

        itemDelete = menu.add(actionDelete);
        itemDelete
                .setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
        popup.add(actionDelete);
        popup.addSeparator();

        menu.addSeparator();

        BPadLogger.logger.debug("Adding Edit toolbar utility");
        m_toolBar.add(new SmallButton(actionCut, myResources.getString("Cut"),
                createIcon("Cut")));
        m_toolBar.add(new SmallButton(actionCopy,
                myResources.getString("Copy"), createIcon("Copy")));
        m_toolBar.add(new SmallButton(actionPaste, myResources
                .getString("Paste"), createIcon("Paste")));

        m_toolBar.addSeparator();

        m_toolBar.add(new SmallButton(m_undoAction, myResources
                .getString("Undo"), createIcon("Undo")));
        m_toolBar.add(new SmallButton(m_redoAction, myResources
                .getString("Redo"), createIcon("Redo")));

        m_toolBar.addSeparator();

        /**
         * TODO: Here we need to modify font(Bengali and English) for find and
         * replace. we will get this in FindDialog.java(line 71)
         */
        Action findAction = new AbstractAction(myResources.getString("Find")
                + "...") {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User Click Find menu");
                if (m_findDialog == null) {
                    BPadLogger.logger
                            .debug("Initializing FindDialog() from VietPad");
                    m_findDialog = new FindDialog(BanglaPad.this, 0);
                } else {
                    m_findDialog.setSelectedIndex(0);
                }
                m_findDialog.setVisible(true);
            }
        };
        item = menu.add(findAction);
        item.setMnemonic('f');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, MENU_MASK));
        m_toolBar.add(new SmallButton(findAction,
                myResources.getString("Find"), createIcon("Find")));

        m_toolBar.addSeparator();

        Action replaceAction = new AbstractAction(myResources
                .getString("Replace")
                + "...") {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User Click Replace menu ");
                if (m_findDialog == null) {
                    BPadLogger.logger
                            .debug("Initializing FindDialog() from VietPad");
                    m_findDialog = new FindDialog(BanglaPad.this, 1);
                } else {
                    m_findDialog.setSelectedIndex(1);
                }
                m_findDialog.setVisible(true);
            }
        };

        item = menu.add(replaceAction);
        item.setMnemonic('l');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, MENU_MASK));

        menu.addSeparator();

        actionSelectAll = new AbstractAction(myResources
                .getString("Select_All"), null) {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click selectAll menu");
                m_editor.selectAll();
            }
        };

        item = menu.add(actionSelectAll);
        item.setMnemonic('a');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, MENU_MASK));
        popup.add(actionSelectAll);

        m_editor.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }

            public void mouseReleased(MouseEvent e) {
                mousePressed(e);
            }
        });

        /**
         * TODO: Here we need to modify date & time format for Bengali support.
         */
        menu
                .add(new AbstractAction(myResources.getString("Date_&_Time"),
                        null) {
                    DateFormat df;

                    public void actionPerformed(ActionEvent e) {
                        BPadLogger.logger.debug("User click Date & Time  menu");
                        if (df == null) {
                            /*
                             * df =
                             * DateFormat.getDateTimeInstance(DateFormat.FULL,
                             * DateFormat.SHORT, VIETNAM);
                             */
                            df = new SimpleDateFormat(
                                    "EEEE,  d  M  yyyy, HH:mm:ss Z", BANGLA);
                        }
                        undoSupport.beginUpdate();
                        m_editor.replaceSelection(df.format(new Date()));
                        undoSupport.endUpdate();
                    }
                });

        menuBar.add(menu);

        // View menu

        menu = new JMenu(myResources.getString("View"));
        menu.setMnemonic('v');

        final JMenuItem viewToolbar = new JMenuItem(toolBarOn ? myResources
                .getString("Hide_Toolbar") : myResources
                .getString("Show_Toolbar"));
        viewToolbar.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click view menu");
                toolBarOn ^= true;
                viewToolbar.setText(toolBarOn ? myResources
                        .getString("Hide_Toolbar") : myResources
                        .getString("Show_Toolbar"));
                m_toolBar.setVisible(toolBarOn);
            }
        });
        viewToolbar.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B,
                MENU_MASK));
        menu.add(viewToolbar);

        menu.addSeparator();

        subMenu = new JMenu(myResources.getString("Look_&_Feel"));

        ActionListener lafLst = new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                BPadLogger.logger.debug("User click Look and Feel menu");
                updateLaF(ae.getActionCommand());
            }
        };

        ButtonGroup groupLookAndFeel = new ButtonGroup();
        UIManager.LookAndFeelInfo[] lafs = UIManager.getInstalledLookAndFeels();
        for (int i = 0; i < lafs.length; i++) {
            JRadioButtonMenuItem lafButton = new JRadioButtonMenuItem(lafs[i]
                    .getName());
            lafButton.setActionCommand(lafs[i].getClassName());
            if (UIManager.getLookAndFeel().getName().startsWith(
                    lafs[i].getName())) {
                lafButton.setSelected(true);
            }
            lafButton.addActionListener(lafLst);
            groupLookAndFeel.add(lafButton);
            subMenu.add(lafButton);
        }

        menu.add(subMenu);
        menuBar.add(menu);

        // Help menu

        menu = new JMenu(myResources.getString("Help"));
        menu.setMnemonic('h');

        item = startJavaHelp();

        menu.add(item);

        if (!MAC_OS_X) {
            menu.addSeparator();
            item = new JMenuItem(myResources.getString("About_")
                    + BPadLogger.APP_NAME);
            item.setMnemonic('A');
            item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    about();
                }
            });
            menu.add(item);
        }

        menuBar.add(menu);
        /**
         * FIXME: Code Added By Zahurul
         */

        fontNameList = new JComboBox();
        fontFamily = GraphicsEnvironment.getLocalGraphicsEnvironment()
                .getAvailableFontFamilyNames();
        for (int i = 0; i < fontFamily.length; i++) {
            if (new Font(fontFamily[i], Font.PLAIN, 12).canDisplay('\u0981')
                    || new Font(fontFamily[i], Font.PLAIN, 12)
                            .canDisplay('\u0065')) {
                fontNameList.addItem(fontFamily[i]);
            }
        }
        fontNameList.setMaximumSize(fontNameList.getPreferredSize());
        ActionListener lst = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fontName = fontNameList.getSelectedItem().toString();
                BPadLogger.logger.debug("User chose: " + fontName + " font");
                MutableAttributeSet attr = new SimpleAttributeSet();
                StyleConstants.setFontFamily(attr, fontName);
                setAttributeSet(attr);
                m_editor.grabFocus();
            }
        };
        fontNameList.addActionListener(lst);
        m_toolBar.add(fontNameList);
        fontNameList.setEditable(true);
        m_toolBar.addSeparator();

        fontSizes = new String[] { "8", "9", "10", "11", "12", "14", "16",
                "18", "20", "22", "24", "26", "28", "36", "48", "72", "96" };
        fontSizeList = new JComboBox(fontSizes);
        fontSizeList.setMaximumSize(fontSizeList.getPreferredSize());
        fontSizeList.setEditable(true);
        lst = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int font_Size = 0;
                try {
                    font_Size = Integer.parseInt(fontSizeList.getSelectedItem()
                            .toString());
                    BPadLogger.logger.debug("User chose: " + font_Size
                            + " font size");
                } catch (NumberFormatException ex) {
                    BPadLogger.logger.debug("Exception:  " + ex.toString()
                            + " occured in VietPad");
                    return;
                }

                fontSize = font_Size;
                MutableAttributeSet attr = new SimpleAttributeSet();
                StyleConstants.setFontSize(attr, font_Size);
                setAttributeSet(attr);
                m_editor.grabFocus();
            }
        };
        fontSizeList.addActionListener(lst);
        m_toolBar.add(fontSizeList);
        m_toolBar.addSeparator();

        BoldButton = new SmallToggleButton(false, createIcon("BoldHK"),
                createIcon("BoldHK"), "Bold font");
        lst = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click Bold button ");
                MutableAttributeSet attr = new SimpleAttributeSet();
                StyleConstants.setBold(attr, BoldButton.isSelected());
                setAttributeSet(attr);
                m_editor.grabFocus();
            }
        };
        BoldButton.addActionListener(lst);
        m_toolBar.add(BoldButton);
        ItalicButton = new SmallToggleButton(false, createIcon("ItalicHK"),
                createIcon("ItalicHK"), "Italic font");
        lst = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click Italic button ");
                MutableAttributeSet attr = new SimpleAttributeSet();
                StyleConstants.setItalic(attr, ItalicButton.isSelected());
                setAttributeSet(attr);
                ItalicButton.grabFocus();
            }
        };
        ItalicButton.addActionListener(lst);
        m_toolBar.add(ItalicButton);

        underLineButton = new SmallToggleButton(false,
                createIcon("UnderlineHK"), createIcon("UnderlineHK"),
                "Italic font");
        lst = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                BPadLogger.logger.debug("User click Underline button ");
                MutableAttributeSet attr = new SimpleAttributeSet();
                StyleConstants.setUnderline(attr, underLineButton.isSelected());
                setAttributeSet(attr);
                underLineButton.grabFocus();
            }
        };
        underLineButton.addActionListener(lst);
        m_toolBar.add(underLineButton);

        getContentPane().add(m_toolBar, BorderLayout.PAGE_START, 0);

        return menuBar;
    }

    // end of createMenuBar()

    private JMenuItem startJavaHelp() {
        JMenuItem mi = null;
        try {
            String aboutURL = getClass().getResource("/org/gjt/sp/jedit/icons/")+"help/BPad.hs"
            .toString();
          URL url = new URL(aboutURL); // replace this with the actual location of your help set
          HelpSet hs = new HelpSet(null, url);
          HelpBroker hb = hs.createHelpBroker();
          mi = new JMenuItem("BanglaPad Help");
          mi.addActionListener(new CSH.DisplayHelpFromSource(hb));
        }
        catch (Exception e) {
          e.printStackTrace();
        }
        return mi;
      }
    
    public void MoreAbout() {
        BPadLogger.logger.debug("User click for More help");
        final String readme = myResources.getString("readme");
        if (MAC_OS_X && new File(readme).exists()) {
            try {
                Runtime.getRuntime().exec(
                        new String[] { "open", "-a", "Help Viewer", readme });
            } catch (IOException x) {
                BPadLogger.logger.debug("Exception:  " + x.toString()
                        + " occured in Banglapad ");
            }
        } else {
            if (helptopicsFrame == null) {
                helptopicsFrame = new JFrame(
                        "About" + " " + BPadLogger.APP_NAME );
                helptopicsFrame.getContentPane().setLayout(new BorderLayout());
                HtmlPane helpPane = new HtmlPane(imagePath + readme);
                helptopicsFrame.getContentPane().add(helpPane,
                        BorderLayout.CENTER);
                helptopicsFrame.getContentPane().add(helpPane.getStatusBar(),
                        BorderLayout.SOUTH);
                helptopicsFrame.pack();
                helptopicsFrame.setLocation((screen.width - helptopicsFrame
                        .getWidth()) / 2, 40);
            }
            helptopicsFrame.setVisible(true);
        }
    }

    /**
     * Gets the textArea attribute of the VietPad object
     * 
     * @return The textArea value
     */
    /**
     * Code change from now the return type of this method is JTextPane
     */
    public JTextPane getTextArea() {
        return m_editor;
    }

    /**
     * Sets the selection attribute of the VietPad object
     * 
     * @param xStart
     *            The start of the selection
     * @param xFinish
     *            The end of the selection
     */
    public void setSelection(int xStart, int xFinish) {
        setSelection(xStart, xFinish, false);
    }

    /**
     * Sets the selection of the text editor
     * 
     * @param xStart
     *            The new selection start
     * @param xFinish
     *            The new selection finish
     * @param moveUp
     */
    public void setSelection(int xStart, int xFinish, boolean moveUp) {
        if (moveUp) {
            m_editor.setCaretPosition(xFinish);
            m_editor.moveCaretPosition(xStart);
        } else {
            m_editor.select(xStart, xFinish);
        }
    }

    /**
     * Gets the selected text of the text editor
     * 
     * @return The selection value
     */
    public String getSelection() {
        return m_editor.getSelectedText();
    }

    /**
     * Creates a new document
     */
    protected void newDocument() {
        BPadLogger.logger
                .debug("Creating new document in newDocument() in VietPad");
        if (currentTmpFolder != null) {
            New.deletePrevTmp(currentTmpFolder);
        }
        currentTmpFolder = New.createTmpDir();

        m_editor.setText(null);
        m_currentFile = null;
        m_doc.putProperty(DefaultEditorKit.EndOfLineStringProperty, System
                .getProperty("line.separator"));
        setTitle(myResources.getString("Untitled") + " - "
                + BPadLogger.APP_NAME);
        m_editor.scrollRectToVisible(new Rectangle(0, 0, 1, 1));
        // m_chooser.setFileFilter(utf8Filter);
        updateSave(false);
        m_undo.discardAllEdits();
        updateUndoRedo();
        updateCutCopyDelete(false);
    }

    String getTmpPath(String dirName) {
        String tmpDir = System.getProperty("java.io.tmpdir") + "/";
        return tmpDir + dirName + File.separator + "document.html";
    }

    /**
     * Export as HTML
     */

    public void exportHTML() {
        // getGlassPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        // getGlassPane().setVisible(true);
        BPadLogger.logger.debug("First Line of exportHTML() in VietPad");
        m_chooser.removeChoosableFileFilter(bngFilter);
        m_chooser.removeChoosableFileFilter(txtFilter);
        m_chooser.removeChoosableFileFilter(utf8Filter);
        m_chooser.removeChoosableFileFilter(utf16Filter);

        m_chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        if (m_chooser.showDialog(BanglaPad.this, "Folder") != JFileChooser.APPROVE_OPTION) {
            BanglaPad.this.setCursor(Cursor
                    .getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            // getGlassPane().setVisible(false);
            return;
        }
        BanglaPad.this.repaint();
        File fChoosen = m_chooser.getSelectedFile();
        try {
            String currentTmpFile = currentTmpFolder + File.separator
                    + "document.html";
            String curTmpFolder = currentTmpFolder.substring(currentTmpFolder
                    .lastIndexOf(File.separator) + 1);
            File curTmpF = new File(currentTmpFile);
            OutputStream out = new FileOutputStream(curTmpF);
            htmlKit.write(out, m_doc, 0, m_doc.getLength());
            out.close();

            Zip z = new Zip();
            String tmpDir = System.getProperty("java.io.tmpdir");
            String absPath = fChoosen.toString();
            String fileNameS = absPath.substring(absPath
                    .lastIndexOf(File.separator) + 1);
            absPath = absPath.substring(0, absPath.lastIndexOf(File.separator));

            z.zipFiles(tmpDir, curTmpFolder, absPath, fileNameS);

            m_chooser.addChoosableFileFilter(bngFilter);
            m_chooser.addChoosableFileFilter(txtFilter);
            m_chooser.addChoosableFileFilter(utf8Filter);
            m_chooser.addChoosableFileFilter(utf16Filter);
            m_chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
            m_chooser.setFileFilter(bngFilter);
            String str = absPath + File.separator + fileNameS;
            absPath += File.separator;
            UnZip.unzip(str, absPath);

            String rF = absPath + curTmpFolder;
            String nF = absPath + fileNameS + "_HTML";
            File renameFile = new File(rF);
            File newFile = new File(nF);
            boolean a;
            a = renameFile.renameTo(newFile);

            rF = absPath + fileNameS;
            File removeFile = new File(rF);
            removeFile.delete();

            String msg = "Your HTML folder has been saved under " + nF
                    + " directory";
            JOptionPane.showMessageDialog(null, msg, "alert",
                    JOptionPane.WARNING_MESSAGE);
            // JOptionPane.showMessageDialog(this,
            // myResources.getString("Cannot_find_file_") + file, APP_NAME,
            // JOptionPane.WARNING_MESSAGE);
        } catch (Exception e) {
            BPadLogger.logger.debug("Exception:  " + e.toString()
                    + " occured in VietPad");
        }
    }

    /**
     * Opens a document
     * 
     * @param file
     *            A file to be opened, null for Open dialog
     */
    public void openDocument(File file) {
        if (currentTmpFolder != null && bngFormat == true) {
            New.deletePrevTmp(currentTmpFolder);
            currentTmpFolder = null;
        }
        getGlassPane()
                .setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        getGlassPane().setVisible(true);
        FileInformation fi_temporary;
        String decoded_contents = ""; // from file
        String temp;
        String script;

        String decision = "";

        if (m_chooser.showOpenDialog(BanglaPad.this) != JFileChooser.APPROVE_OPTION) {
            BanglaPad.this.setCursor(Cursor
                    .getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            getGlassPane().setVisible(false);
            return;
        }
        BPadLogger.logger
                .debug("Calling newDocument() form openDocument() in VietPad");
        newDocument();

        BanglaPad.this.repaint();
        File fChoosen = m_chooser.getSelectedFile();
        m_currentFile = fChoosen;

        setTitle(m_currentFile.getName() + " - " + BPadLogger.APP_NAME);
        fi_temporary = Detect.whatIsIt(fChoosen);

        if (fi_temporary == null
                || fi_temporary.status == FileInformation.CANT_LOAD) {
            JOptionPane.showMessageDialog(this, myResources
                    .getString("Cannot_find_file_")
                    + file, BPadLogger.APP_NAME, JOptionPane.WARNING_MESSAGE);
            return;
        }
        fi_temporary.fileName = fChoosen.getPath();
        try {
            if (m_chooser.getFileFilter() == bngFilter) {
                bngFormat = true;
                // loadHTML(fChoosen);
                BPadLogger.logger
                        .debug("Calling UnZip.unzip() from openDocument() in VietPad");
                String dirName = UnZip.unzip(m_currentFile.toString(), null);
                fChoosen = new File(getTmpPath(dirName));
                dirName = fChoosen.toString();
                dirName = dirName.substring(0, dirName
                        .lastIndexOf(File.separator));
                currentTmpFolder = dirName;
                BPadLogger.logger
                        .debug("Calling readChar() from openDocument() in VietPad");
                decoded_contents = readChars(fChoosen, null);
                StringReader reader = new StringReader(decoded_contents);
                try {
                    htmlKit.read(reader, m_doc, 0);
                } catch (Exception e) {
                    BPadLogger.logger.debug("Exception:  " + e.toString()
                            + " occured in VietPad");
                }

                try {
                    m_doc.setBase(new URL("file:" + currentTmpFolder
                            + File.separator));
                } catch (Exception e) {
                    BPadLogger.logger.error("problem with setting base" + e);
                }

            } else {
                bngFormat = false;
                if (fi_temporary.encoding.startsWith("UTF-16")) {
                    decoded_contents = readChars(fChoosen,
                            fi_temporary.encoding);
                } else if (fi_temporary.encoding.equals("UTF8")) {
                    decoded_contents = readChars(fChoosen,
                            fi_temporary.encoding);
                } else {
                    decoded_contents = readChars(fChoosen, "ISO8859_1");
                }
                decoded_contents = decoded_contents.replaceAll("\n", "<br>");
                StringReader reader = new StringReader(decoded_contents);
                htmlKit.read(reader, m_doc, 0);
            }
        } catch (OutOfMemoryError e) {
            BPadLogger.logger.debug("Exception:  " + e.toString()
                    + " occured in VietPad");
            JOptionPane
                    .showMessageDialog(
                            this,
                            BPadLogger.APP_NAME
                                    + myResources
                                            .getString("_has_run_out_of_memory.\nPlease_restart_")
                                    + BPadLogger.APP_NAME
                                    + myResources.getString("_and_try_again."),
                            myResources.getString("Out_of_Memory"),
                            JOptionPane.ERROR_MESSAGE);
        } catch (Exception ex) {
            showError(ex, myResources.getString("Error_reading_file_")
                    + m_currentFile);
        } finally {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    m_editor.setCaretPosition(0);
                    m_editor.scrollRectToVisible(new Rectangle(0, 0, 1, 1));
                    updateSave(false);
                    m_undo.discardAllEdits();
                    updateUndoRedo();
                    updateCutCopyDelete(false);
                    getGlassPane().setCursor(
                            Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                    getGlassPane().setVisible(false);
                }
            });
        }
        BPadLogger.logger.debug("End of openDocument() in VietPad");
    }

    protected String readChars(File file_to_open, String encoding) {
        char[] buffer;
        int maxSize = (int) file_to_open.length();
        long numberRead;

        InputStreamReader fin;

        try {
            if (encoding == null) {
                fin = new InputStreamReader(new FileInputStream(file_to_open));
            } else {
                fin = new InputStreamReader(new FileInputStream(file_to_open),
                        encoding);
            }
        } catch (FileNotFoundException fnf) {
            BPadLogger.logger.debug("readChars: File not found. "
                    + fnf.toString());
            return "";
        } catch (UnsupportedEncodingException uex) {
            BPadLogger.logger.debug("readChars: Unknown encoding. "
                    + uex.toString());
            return "";
        }

        try {
            buffer = new char[maxSize];
            numberRead = fin.read(buffer, 0, maxSize);
        } catch (Exception e) {
            BPadLogger.logger.debug("readChars: Error while reading.\n"
                    + e.toString());
            return "";
        }
        if (numberRead == 0)
            return "";
        BPadLogger.logger.debug("End of readChars() in VietPad");
        // Remove first character for UTF-16
        if (numberRead > 1 && buffer[0] == 0xFEFF
                && encoding.startsWith("UTF-16", 0)) {
            return new String(buffer, 1, (int) numberRead - 1);
        } else {
            return new String(buffer, 0, (int) numberRead);
        }
    }

    /*
     * void loadHTML (File file_to_open) {
     * 
     * m_doc = (HTMLDocument)htmlKit.createDefaultDocument();
     * 
     * try { FileInputStream fin; fin = new FileInputStream(file_to_open);
     * htmlKit.read(fin, m_doc, 0); fin.close(); } catch (BadLocationException
     * ble) { System.err.println("loadRTF: Bad Location.\n" + ble.getMessage());
     * ble.printStackTrace(); } catch (IOException io) {
     * System.err.println("loadRTF: I/O error. \n" + io.getMessage());
     * io.printStackTrace(); } m_editor.setDocument(m_doc); //
     * m_editor.setStyledDocument(m_doc); m_editor.requestFocusInWindow();
     * m_editor.validate(); } // load HTML
     */

	/**
	 * Saves document to file
	 * 
	 * @param saveAs
	 *            True for save as
	 *            False for save
	 * @return True for success
	 */
	protected boolean saveFile(boolean saveAs) {

		BPadLogger.logger.debug("Inside saveAs, BanglaPad with saveAs val: "+saveAs);
		Segment segment;
		String absPath = ".";/*absPath is the user's save directory*/
		String fileNameS = null;/*fileNameS is the user given file name*/
		/**FIXME: Remove FileInformation().encoding. We can assign null to it.
		 */
		String decision = new FileInformation().encoding;
		if (!saveAs && !m_textChanged){
			BPadLogger.logger.debug("saveAs:"+saveAs+" m_textChanged"+m_textChanged+" Exiting from save function");
			return true;
		}
		/*
		 * m_currentFile is a File object created for current file. Initially it is null
		 */
		if (saveAs || m_currentFile == null) {
			BPadLogger.logger.debug("saveAs:"+saveAs+" OR m_currentFile==null. Save method, BanglaPad");
			if (m_chooser.showSaveDialog(BanglaPad.this) != JFileChooser.APPROVE_OPTION)
				return false; /*Returns when user press Cancel*/
			/**FIXME: Does f ever get a null value? Issue#BPAD2*/
			File f = m_chooser.getSelectedFile();
			if (f == null)
				return false;
			String extension = ".bng";
			/**FIXME: Equality of FileFilter object. Issue#BPAD1. Resolution: Solved*/
			if (m_chooser.getFileFilter().equals(bngFilter) == true) {
				extension = ".bng";
			} else {
				extension = ".txt";
			}
			BPadLogger.logger.debug("Extension is: "+extension);
			m_currentFile = f.getName().indexOf('.') > -1 ? f : new File(
					m_chooser.getCurrentDirectory(), f.getName() + extension);
			/**FIXME: Change BPadLogger.APP_NAME! Get the name from any other place not BPadLogger. 
			 * 
			 */
			String title = m_currentFile.getName() + " - " + BPadLogger.APP_NAME; 
			setTitle(title);
			BPadLogger.logger.debug("Set the title:"+title);
			fileNameS = m_currentFile.getName();/*fileNameS is the user given file name*/
			BPadLogger.logger.debug("User given file name:"+fileNameS);
			absPath = f.toString();
			absPath = absPath.substring(0, absPath.lastIndexOf(File.separator));/*absPath is the user's save directory*/
			BPadLogger.logger.debug("User's save directory:"+absPath);
		}

		if (!saveAs && !m_textChanged)
            return true;
        if (saveAs || m_currentFile == null) {
            if (m_chooser.showSaveDialog(BanglaPad.this) != JFileChooser.APPROVE_OPTION)
                return false;
            File f = m_chooser.getSelectedFile();
            if (f == null)
                return false;
            String extension = ".bng";
            if (m_chooser.getFileFilter() == bngFilter) {
                extension = ".bng";
            } else {
                extension = ".txt";
            }
            m_currentFile = f.getName().indexOf('.') > -1 ? f : new File(
                    m_chooser.getCurrentDirectory(), f.getName() + extension);
            setTitle(m_currentFile.getName() + " - " + BPadLogger.APP_NAME);
            fileNameS = m_currentFile.getName();
            absPath = f.toString();
            absPath = absPath.substring(0, absPath.lastIndexOf(File.separator));
        }

        BanglaPad.this
                .setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        try {
			/**FIXME: Equality of FileFilter object. Issue#BPAD1. Resolution: Solved*/
			if (m_chooser.getFileFilter().equals(bngFilter) == true) {
				bngFormat = true;/**FIXME:Check if we need this bngFormat boolean later.*/
				/*currentTmpFolder is the systemTmpPath+TimeStamp*/
				/*currentTmpFile is the absolute path of document.html under tmp directory*/
				String currentTmpFile = currentTmpFolder + File.separator
						+ "document.html";
				/*curTmpFolder is the timeStamp*/
				String curTmpFolder = currentTmpFolder
						.substring(currentTmpFolder.lastIndexOf(File.separator) + 1);
				BPadLogger.logger.debug("Current Temp folder(with timeStamp):"+currentTmpFolder);
				BPadLogger.logger.debug("Absolute path of document.html under system tmp:"+currentTmpFile);
				BPadLogger.logger.debug("Time Stamp"+curTmpFolder);
				/*curTmpF is the File object of tmp document file*/
				File curTmpF = new File(currentTmpFile);
				OutputStream out = new FileOutputStream(curTmpF);
				htmlKit.write(out, m_doc, 0, m_doc.getLength());
				out.close();
				BPadLogger.logger
						.debug("Initializing Zip() from saveFile() in BanglaPad");
				Zip z = new Zip();
				String tmpDir = System.getProperty("java.io.tmpdir");/**Unnecessary variable, remove later..*/
				/**
				 * tmpDir - System tmp directory
				 * curTmpFolder - timeStamp
				 * absPath - User's save directory
				 * fileNameS - User given file name
				 */
				BPadLogger.logger.debug("Sending to zip with System tmp dir:"+tmpDir+";timeStamp:"+curTmpFolder+";user save dir:"+absPath+";user file name"+fileNameS);
				z.zipFiles(tmpDir, curTmpFolder, absPath, fileNameS);
			} else {
				bngFormat = false;
				String currentTmpFile = absPath + File.separator + fileNameS;
				File curTmpF = new File(currentTmpFile);
				OutputStream out = new FileOutputStream(curTmpF);
				if (m_chooser.getFileFilter().equals(utf16Filter) == true) {
					decision = "UTF16";
				} else {
					decision = "UTF8";
				}
				OutputStreamWriter w;
				String str = m_doc.getText(0, m_doc.getLength());
				str = str.replaceAll("<br>", "\n");
				// segment = getSegment(m_doc);
				w = new OutputStreamWriter(out, decision);
				if (decision.startsWith("UTF-16"))
					w.write(0xFEFF);	/*BOM, Byte Order Mark*/
				// w.write(segment.array, segment.offset, segment.count);
				w.write(str);
				w.close();
			}

            if (m_chooser.getFileFilter() == bngFilter) {
                bngFormat = true;
                String currentTmpFile = currentTmpFolder + File.separator
                        + "document.html";
                String curTmpFolder = currentTmpFolder
                        .substring(currentTmpFolder.lastIndexOf(File.separator) + 1);
                File curTmpF = new File(currentTmpFile);
                OutputStream out = new FileOutputStream(curTmpF);
                htmlKit.write(out, m_doc, 0, m_doc.getLength());
                out.close();
                BPadLogger.logger
                        .debug("Initializing Zip() from saveFile() in VietPad");
                Zip z = new Zip();
                String tmpDir = System.getProperty("java.io.tmpdir");
                z.zipFiles(tmpDir, curTmpFolder, absPath, fileNameS);
            } else {
                bngFormat = false;
                String currentTmpFile = absPath + File.separator + fileNameS;
                File curTmpF = new File(currentTmpFile);
                OutputStream out = new FileOutputStream(curTmpF);
                if (m_chooser.getFileFilter() == utf16Filter) {
                    decision = "UTF16";
                } else {
                    decision = "UTF8";
                }
                OutputStreamWriter w;
                String str = m_doc.getText(0, m_doc.getLength());
                str = str.replaceAll("<br>", "\n");
                // segment = getSegment(m_doc);
                w = new OutputStreamWriter(out, decision);
                if (decision.startsWith("UTF-16"))
                    w.write(0xFEFF);
                // w.write(segment.array, segment.offset, segment.count);
                w.write(str);
                w.close();
            }

            // m_textChanged = false;
            updateSave(false);

        } catch (OutOfMemoryError e) {
            e.printStackTrace();
            JOptionPane
                    .showMessageDialog(
                            this,
                            BPadLogger.APP_NAME
                                    + myResources
                                            .getString("_has_run_out_of_memory.\nPlease_restart_")
                                    + BPadLogger.APP_NAME
                                    + myResources.getString("_and_try_again."),
                            myResources.getString("Out_of_Memory"),
                            JOptionPane.ERROR_MESSAGE);
        } catch (FileNotFoundException fnfe) {
            showError(fnfe, myResources.getString("Error_saving_file_")
                    + m_currentFile
                    + myResources.getString(".\nFile_is_inaccessible."));
        } catch (Exception ex) {
            showError(ex, myResources.getString("Error_saving_file_")
                    + m_currentFile);
        } finally {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    getGlassPane().setCursor(
                            Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                    getGlassPane().setVisible(false);
                }
            });
        }
        BanglaPad.this.setCursor(Cursor
                .getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        BPadLogger.logger.debug("End of saveFile in VietPad");
        return true;

    }

    /*
     * Segment getSegment(Document doc) { Segment segment = new Segment(); try {
     * if (doc != null) doc.getText(0, doc.getLength(), segment); } catch
     * (BadLocationException bad) { System.err.println("Segment error. Eraro de
     * segmento. -E- \n" + bad.getMessage()); } return segment; }
     */

    /**
     * Determines whether a string contains Unicode characters
     * 
     * @param str
     *            The string to investigate
     * @return true if Unicode characters detected
     */
    /**
     * FIXME: Code Modification This method filtering the unicodes.
     */
    protected boolean isUnicode(String str) {
        return !str.matches("[\u0000-\u00FF\u2019]*");
        // ISO Latin 1 plus right single quotation mark
    }

    /**
     * Auto-detects file encoding.
     * 
     * Specifically tailored for detecting Vietnamese Unicode characters. Detect
     * file encodings: Windows Latin 1, UTF-8, -16, -16BE, -16LE. Vietnamese
     * legacy encodings are returned as Windows Latin 1.
     * 
     * @param file
     *            Description of the Parameter
     * @return the encoding detected
     */
    /*
     * protected String detectEncoding(File file) {
     * 
     * String encoding = "UTF8"; // UTF-8 as default encoding long filesize =
     * file.length();
     * 
     * if (filesize == 0) { return encoding; } if (filesize > 1024) { filesize =
     * 1024; // read in 1024 bytes, increase if necessary } int[] b = new
     * int[(int) filesize];
     * 
     * try { DataInputStream in = new DataInputStream(new
     * FileInputStream(file)); for (int i = 0; i < filesize; i++) { b[i] =
     * in.readUnsignedByte(); } in.close();
     * 
     * if (b[0] == 0xFE && b[1] == 0xFF) { // UTF-16 and -16BE BOM FEFF encoding =
     * "UTF-16"; } else if (b[0] == 0xFF && b[1] == 0xFE) { // UTF-16LE BOM FFFE
     * encoding = "UTF-16LE"; } else if (b[0] == 0xEF && b[1] == 0xBB && b[2] ==
     * 0xBF) { encoding = "UTF8"; } else { for (int i = 0; i < b.length; i++) {
     * if ((b[i] & 0xF0) == 0xC0) { if ((b[i + 1] & 0x80) == 0x80) { encoding =
     * "UTF8"; break; } } else if ((b[i] & 0xE1) == 0xE1) { if ((b[i + 1] &
     * 0x80) == 0x80 && (b[i + 2] & 0x80) == 0x80) { encoding = "UTF8"; break; } }
     * else if ((b[i] & 0x80) != 0x00) { encoding = "Cp1252"; } } } } catch
     * (IOException e) {
     * System.err.println(myResources.getString("Can't_open_file.")); } catch
     * (Exception e) { System.err.println(e.getMessage()); }
     * 
     * return encoding; }
     */

    /**
     * Display a dialog to save changes
     * 
     * @return false if user canceled, true else
     */
    protected boolean promptToSave() {
        if (!m_textChanged) {
            return true;
        }
        switch (JOptionPane.showConfirmDialog(this, myResources
                .getString("Do_you_want_to_save_the_changes_to")
                + "\n\""
                + (m_currentFile == null ? myResources.getString("Untitled")
                        : m_currentFile.getName()) + "\"?",
                BPadLogger.APP_NAME, JOptionPane.YES_NO_CANCEL_OPTION,
                JOptionPane.INFORMATION_MESSAGE)) {
        case JOptionPane.YES_OPTION:
            return saveFile(false);
        case JOptionPane.NO_OPTION:
            return true;
        default:
            return false;
        }
    }

    /**
     * Shows a warning message
     * 
     * @param e
     *            the exception to warn about
     * @param message
     *            the message to display
     */
    public void showError(Exception e, String message) {
        JOptionPane.showMessageDialog(this, message, BPadLogger.APP_NAME,
                JOptionPane.WARNING_MESSAGE);
    }

    protected void changeContext() {
        if (banglaModeOn) {
            UserKeyMap.loadUserKeyMap(new File("keymaps/UI_bengali.kmp")); // initialize
                                                                            // tables
            UserKeyMap.setEnabled(true);
            fontNameList.setSelectedItem("Mukti Narrow");
            keybordMenu.setEnabled(true);
        } else {
            fontNameList.setSelectedItem("Dialog");
            UserKeyMap.loadUserKeyMap(null); // initialize tables
            keybordMenu.setEnabled(false);
        }

    }

    /**
     * Updates the Save action
     * 
     * @param modified
     *            whether file has been modified
     */
    private void updateSave(boolean modified) {
        if (m_textChanged != modified) {
            m_textChanged = modified;
            actionSave.setEnabled(modified);
            rootPane.putClientProperty("windowModified", Boolean
                    .valueOf(modified));
            // see http://developer.apple.com/qa/qa2001/qa1146.html
        }
    }

    /**
     * Updates the Undo and Redo actions
     */
    private void updateUndoRedo() {
        m_undoAction.setEnabled(m_undo.canUndo());
        m_redoAction.setEnabled(m_undo.canRedo());
    }

    /**
     * Updates the Cut, Copy, and Delete actions
     * 
     * @param isTextSelected
     *            whether any text currently selected
     */
    private void updateCutCopyDelete(boolean isTextSelected) {
        actionCut.setEnabled(isTextSelected);
        actionCopy.setEnabled(isTextSelected);
        actionDelete.setEnabled(isTextSelected);
    }

    /**
     * Updates the Paste action
     */
    private void updatePaste() {
        Transferable clipData = clipboard.getContents(clipboard);
        if (clipData != null) {
            actionPaste.setEnabled(clipData
                    .isDataFlavorSupported(DataFlavor.stringFlavor));
        }
    }

    /**
     * Updates UI component if changes in LAF
     * 
     * @param laf
     *            the look and feel class name
     */
    protected void updateLaF(String laf) {
        try {
            UIManager.setLookAndFeel(laf);
        } catch (Exception exc) {
            BPadLogger.logger.debug("Exception:  " + exc.toString()
                    + " occured in VietPad");
            // do nothing
        }
        SwingUtilities.updateComponentTreeUI(this);
        SwingUtilities.updateComponentTreeUI(popup);
        SwingUtilities.updateComponentTreeUI(m_chooser);

        if (m_findDialog != null) {
            SwingUtilities.updateComponentTreeUI(m_findDialog);
        }
    }

    /**
     * Displays Page Setup dialog
     */
    protected void pageSetup() {
        // to be implemented in subclass
    }

    /**
     * Displays Print dialog
     */
    protected void printPage() {
        // to be implemented in subclass
    }

    /**
     * Displays the About box
     */
protected void about() {
        String dialogInfo = BPadLogger.APP_NAME + ", v0.1 \u00a9 2005\n" +
                "Bangla Unicode Rich Text Editor\n" +
                DateFormat.getDateInstance(DateFormat.LONG)
                .format(new GregorianCalendar(2005, Calendar.NOVEMBER, 20).getTime()) +
                "\nhttp://sourceforge.net/projects/banglapad\n" +
                "\nTeam members:\n" +
                "Zahurul Islam\nNaushad UzZaman\nMatin Saad Abdullah\nMumit Khan\n" +
                "\nAffiliation:\n" +
                "Center for Research on Bangla Language Processing\n" +
                "BRAC University, Dhaka, Bangladesh\n" +
                "\nAcknowledgement: \n"+
                "Vietpad http://vietpad.sourceforge.net\n" +
                "Simredo for Keymap\n"+
                "Swing 2 by Matthew Robinson and Pavel Vorobiev, for Rich Text Editing\n"
                ;
        String jB [] = new String[2];
        jB[0] = "OK";
        jB[1] = "More";
        JOptionPane jop = new JOptionPane(dialogInfo, JOptionPane.INFORMATION_MESSAGE,
                JOptionPane.YES_NO_CANCEL_OPTION,
                null, jB);
        JDialog jopDialog = jop.createDialog(null, "TitleText");
        jopDialog.show();
            
        Object userSelection = jop.getValue();
        System.out.println("user Select == "+userSelection);
        if(userSelection.toString().compareTo("More")==0){
            System.out.println("user Select == "+userSelection);
            MoreAbout();
        }
    }
    /**
     * Quits the application
     */
    protected void quit() {
        DateFormat df;
        df = new SimpleDateFormat("EEEE,  d  M  yyyy, HH:mm:ss Z", Locale
                .getDefault());
        BPadLogger.logger.info("------------------ Stop logging from  :  "
                + df.format(new Date()) + "  ------------------");
        if (promptToSave()) {
            prefs.put("currentDirectory", m_chooser.getCurrentDirectory()
                    .getPath());
            prefs.put("fontName", m_font.getName());
            prefs.put("lookAndFeel", UIManager.getLookAndFeel().getClass()
                    .getName());
            prefs.put("inputMethodLocale", getInputContext().getLocale()
                    .toString());
            prefs.putBoolean("smartMark", smartMarkOn);
            prefs.putBoolean("toolBar", toolBarOn);
            prefs.putBoolean("vietMode", banglaModeOn);
            prefs.putBoolean("defaultEOL", defaultEOLOn);
            prefs.putInt("fontSize", m_font.getSize());
            prefs.putInt("fontStyle", m_font.getStyle());
            prefs.putInt("windowState", getExtendedState());

            if (getExtendedState() == NORMAL) {
                prefs.putInt("frameHeight", getHeight());
                prefs.putInt("frameWidth", getWidth());
                prefs.putInt("frameX", getX());
                prefs.putInt("frameY", getY());
            }

            javax.swing.filechooser.FileFilter[] filters = m_chooser
                    .getChoosableFileFilters();
            for (int i = 0; i < filters.length; i++) {
                if (filters[i] == m_chooser.getFileFilter()) {
                    prefs.putInt("filterIndex", i);
                    break;
                }
            }
            if (bngFormat == true)
                New.deleteFolder(currentTmpFolder);
            System.exit(0);
        }
    }

    /**
     * Listens to raw undoable edits
     * 
     * @author Gero Herrmann
     * @created October 26, 2003
     */
    private class RawListener implements UndoableEditListener {
        /**
         * Description of the Method
         * 
         * @param e
         *            Description of the Parameter
         */
        public void undoableEditHappened(UndoableEditEvent e) {
            undoSupport.postEdit(e.getEdit());
        }
    }

    /**
     * Listens to undoable edits filtered by undoSupport
     * 
     * @author Gero Herrmann
     * @created October 26, 2003
     */
    private class SupportListener implements UndoableEditListener {
        /**
         * Description of the Method
         * 
         * @param e
         *            Description of the Parameter
         */
        public void undoableEditHappened(UndoableEditEvent e) {
            updateSave(true);
            m_undo.addEdit(e.getEdit());
            updateUndoRedo();
        }
    }

    private class SmallToggleButton extends JToggleButton implements
            ItemListener {
        protected Border m_raised;

        protected Border m_lowered;

        public SmallToggleButton(boolean selected, ImageIcon imgUnselected,
                ImageIcon imgSelected, String tip) {
            super(imgUnselected, selected);
            setHorizontalAlignment(CENTER);
            setBorderPainted(true);
            m_raised = new BevelBorder(BevelBorder.RAISED);
            m_lowered = new BevelBorder(BevelBorder.LOWERED);
            setBorder(selected ? m_lowered : m_raised);
            setMargin(new Insets(1, 1, 1, 1));
            setToolTipText(tip);
            setRequestFocusEnabled(false);
            setSelectedIcon(imgSelected);
            addItemListener(this);
        }

        public float getAlignmentY() {
            return 0.5f;
        }

        public void itemStateChanged(ItemEvent e) {
            setBorder(isSelected() ? m_lowered : m_raised);
        }
    }

    /**
     * JTextArea with Anti-Aliasing for font size greater than 18. AA in Java
     * can be problematic, however, as the caret and text selection appear not
     * to be at the same exact position.
     * 
     * @author Quan Nguyen
     * @created October 26, 2003
     */

    private int snap(final int ideal, final int min, final int max) {
        final int TOLERANCE = 0;
        return ideal < min + TOLERANCE ? min : (ideal > max - TOLERANCE ? max
                : ideal);
    }
}
