package net.sf.banglapad;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.*;
import java.nio.channels.*;
import java.text.BreakIterator;
import java.util.*;
import javax.swing.*;
import javax.swing.text.BadLocationException;
import com.swabunga.spell.engine.*;
import com.swabunga.spell.swing.JTextComponentSpellChecker;
/**
 *BanglaPadWithTools  
 *@author     Naushad UzZaman (naushadzaman@gmail.com)
 *@author     Zahurul Islam (zahurul@bracuniversity.net)
 *@created    August 10, 2005
 *@version    0.1, December 10, 2005
 *
 *Modified from VietPad
 *@author     Quan Nguyen
 *@author     Gero Herrmann
 *@created    February 11, 2002
 *@version    1.6, 25 May 2005
 */
public class BanglaPadWithTools extends BanglaPadWithFormat
{
    private PreferencesDialog preferencesDlg;
    private Map map;
    private long mapLastModified = Long.MIN_VALUE;
    //static Reader phonetReader = null;
    static Reader engPhonetReader = null;
    static Reader bngPhonetReader = null;
    //static Reader mainDictReader = null;
    static Reader engDictReader = null;
    static Reader bngDictReader = null;
    static SpellDictionary dictionary = null;
    static SpellDictionary engDictionary = null;
    static SpellDictionary bngDictionary = null;
    static SpellDictionary customDictionary = null;
    static File customDictFile = null;
    

    /**
     *  Creates a new instance of BanglaPadWithTools
     */
    public BanglaPadWithTools() {
        super();        
        menuBar.add(createToolsMenu(), menuBar.getMenuCount() - 1);
        changeContext();
        
    
    
        if (MAC_OS_X) {
            new MacOSXApplication(this);
        }
        setVisible(true);
    }
    
    /**
     *  Creates the Tools menu
     *
     *@return    The menu
     */
    private JMenu createToolsMenu() {
        JMenu mTools = new JMenu(myResources.getString("Tools"));
        mTools.setMnemonic('T');
        
        Action spellAction =
            new AbstractAction(myResources.getString("Spell_Check") + "...")
            {
            public void actionPerformed(ActionEvent e) {
                boolean language = banglaModeOn;
                BPadLogger.logger.info("Spelling Checker button clicked");
                BPadLogger.logger.info("Trying to create custom dictionary, load lexicon and phonetic file");
                Object[] options = {"Bangla dictionary", "English dictionary"};
                int n = JOptionPane.showOptionDialog(BanglaPadWithTools.this,
        "Select the dictionary to check spelling " ,
        "Dictionary Selection",
        JOptionPane.YES_NO_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        null,
        options,
        options[0]);
    System.out.println("n value:"+n);
    if(n==0)language = true;
    else if(n==1)language = false;
                try {
                    BPadLogger.logger.debug("User selected the Bangla dictionary");
                    File customDictFile = new File(BPadLogger.supportDir, "custom.dic");
                    
                    if (!customDictFile.exists()) {
                        customDictFile.createNewFile();
                    }
                    BPadLogger.logger.info("Created custom dictionary");
                    /**
                     * FIXME: Added dictionary and phonetic file for Bangla. 
                     * Hard coded these file names. Later will get this file
                     * location from properties file. 
                     */
                    //String phonetFile = "dict/phonet.bn";//"dict/phonet.vi";//
                    String dictFile = "";
                    
                    /*
                    if(phonetReader == null)
                        phonetReader = new InputStreamReader(ClassLoader.getSystemResourceAsStream(phonetFile), "UTF-8");
                    //System.out.println("SpellCheck reader ==  "+phonetReader.toString());
                    BPadLogger.logger.info("Loaded phonetic file");
                    */
                    
                    //Editor in Bangla mode
                    if(language == true) {
                        dictFile = "dict/bn_BD.dic";//"dict/vi_VN.dic";//
                        if(bngDictReader==null) {
                            BPadLogger.logger.debug("Loading Bangla dictionary for the first time");
                            bngDictReader = new InputStreamReader(ClassLoader.getSystemResourceAsStream(dictFile), "UTF-8");
                            BPadLogger.logger.info("Loaded Bangla lexicon file");
                        }
                        else {
                            BPadLogger.logger.info("Bangla lexicon already loaded!");
                        }
                        
                        if(bngDictionary == null) {
                            BPadLogger.logger.info("Send the Bangla lexicon and phonetic file to create a dictionary object");
                            bngDictionary = new SpellDictionaryMultiLingual(bngDictReader, bngPhonetReader, banglaModeOn);
                            BPadLogger.logger.info("Loaded Bangla SpellDictionary object!");
                        }
                        else {
                            BPadLogger.logger.info("Bangla SpellDictionary already loaded!");
                        }
                        
                        dictionary = bngDictionary;
                    }
                    
                    else if(language == false) {
                        dictFile = "dict/english.dic";//
                        if(engDictReader==null) {
                            BPadLogger.logger.debug("Loading English dictionary for the first time");
                            engDictReader = new InputStreamReader(ClassLoader.getSystemResourceAsStream(dictFile), "UTF-8");
                            BPadLogger.logger.info("Loaded English lexicon file");
                        }
                        else {
                            BPadLogger.logger.info("English lexicon already loaded!");
                        }
                        
                        if(engDictionary == null) {
                            BPadLogger.logger.info("Send the English lexicon and phonetic file to create a dictionary object");
                            engDictionary = new SpellDictionaryMultiLingual(engDictReader, engPhonetReader, banglaModeOn);
                            BPadLogger.logger.info("Loaded Bangla SpellDictionary object!");
                        }
                        else {
                            BPadLogger.logger.info("Bangla SpellDictionary already loaded!");
                        }
                        
                        dictionary = engDictionary;
                    }
                     
                    BPadLogger.logger.info("Send the custom lexicon and phonetic file to create a custom dictionary object");
                    customDictionary = new SpellDictionaryMultiLingual(customDictFile, bngPhonetReader, banglaModeOn);                      
                    
                    undoSupport.beginUpdate();
                    
                    new Thread() {
                        public void run() {
                            try {
                                BPadLogger.logger.info("Trying to create JTextComponentSpellChecker object with textArea as parameter");
                                JTextComponentSpellChecker sc = new JTextComponentSpellChecker(dictionary, customDictionary, myResources.getString("Spell_Check"));
                                BPadLogger.logger.info("Checking spelling.. Sending m_editor for checking spelling, BanglaPadWithTools");
                                sc.spellCheck(m_editor);
                            } catch (Exception ex) {
                                ex.printStackTrace();
                            }
                        }
                    }.start();
                    
                    undoSupport.endUpdate();
                    //phonetReader.close();
                    //mainDictReader.close();
                } catch (Exception ex) {
                    ex.printStackTrace();
                    BPadLogger.logger.info("Failed to create custom dictionary or loading lexicon or phonetic file");
                }
            }
            };
            
            JMenuItem item = mTools.add(spellAction);
//          item.setMnemonic('s');
            item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F7, 0));
            
            
            /**
             * FIXME: From here the preferences() method is called
             */
            if (!MAC_OS_X) {
                mTools.addSeparator();
                
                Action optionsAction =
                    new AbstractAction(myResources.getString("Preferences"))
                    {
                    public void actionPerformed(ActionEvent e) {
                        BPadLogger.logger.debug("Spelling Checker button clicked");
                        BPadLogger.logger.debug("Before calling preferences() method.");
                        preferences();
                    }
                    };
                    item = mTools.add(optionsAction);
                    item.setMnemonic('r');
            }
            
            return mTools;
    }
    
    /**
     *  Retrieves options values from Preferences Dialog
     */
    protected void retrieveOptions() {
        BPadLogger.logger.debug("Inside the retrieveOptions() in BanglaPadWithTools");
        defaultEOLOn = preferencesDlg.isDefaultEOL();
        localeBanglaOn = preferencesDlg.isLocaleVN();
        System.out.println("dfaultEOLOn ==  "+defaultEOLOn);
        System.out.println("localeBanglaOn ==  "+localeBanglaOn);
    }
    
    
    /**
     *  Updates UI component if changes in LAF
     *
     *@param  laf  The look and feel class name
     */
    protected void updateLaF(String laf) {
        super.updateLaF(laf);
        
        if (preferencesDlg != null) {
            SwingUtilities.updateComponentTreeUI(preferencesDlg);
            preferencesDlg.pack();
        }
    }
    
    
    /**
     *  Preferences (Options) settings
     */
    
    public void preferences() {
        if (preferencesDlg == null) {
            BPadLogger.logger.debug("Initializing PreferencesDialog Class");
            preferencesDlg = new PreferencesDialog(BanglaPadWithTools.this, true);// modal
            BPadLogger.logger.debug("Before calling preferencesDlg.setDefaultEOL()");
            preferencesDlg.setDefaultEOL(defaultEOLOn);
            BPadLogger.logger.debug("Before calling preferencesDlg.setLocaleVN()");
            preferencesDlg.setLocaleVN(localeBanglaOn);
            
            preferencesDlg.setLocation(
                    prefs.getInt("optionsX", preferencesDlg.getX()),
                    prefs.getInt("optionsY", preferencesDlg.getY()));
        }
        preferencesDlg.setVisible(true);
    }
    
    
    /**
     *  Remembers settings and dialog locations, then quits
     */
    protected void quit() {
        
        if (preferencesDlg != null) {
            prefs.putBoolean("localeVN", localeBanglaOn);
            prefs.putInt("optionsX", preferencesDlg.getX());
            prefs.putInt("optionsY", preferencesDlg.getY());
            
        }
        super.quit();
    } 
    
    
    /**
     *  Starts BanglaPad when called from a Mac OS X application bundle
     *
     *@param  args  The command line arguments
     */
    public static void main(String[] args) {
        new BPadLogger();
        BPadLogger.logger.debug("Initialize the splash screen Launcher");
        new Launcher();    
        BPadLogger.logger.debug("splash screen Launcher Initialization have finished");
        BPadLogger.logger.debug("Initialize the BanglaPadWithTools");
        new BanglaPadWithTools();
        BPadLogger.logger.debug("Close the splash screen");
        EventQueue.invokeLater( new SplashScreenCloser());
    }
    
}
