/*
 *  Copyright 1999-2002 Matthew Robinson and Pavel Vorobiev.
 *  All Rights Reserved.
 *
 *  ===================================================
 *  This program contains code from the book "Swing"
 *  2nd Edition by Matthew Robinson and Pavel Vorobiev
 *  http://www.spindoczine.com/sbe
 *  ===================================================
 */

package net.sf.banglapad;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

/**
 *  Font Dialog
 *@author     Zahurul Islam (zahurul@bracuniversity.net)
 *@version    0.1, December 10, 2005
 *@see        http://sourceforge.net/projects/banglapad
 *
 *Modified from VietPad
 *@author     Quan Nguyen
 *@version    1.0.7, April 14, 2004
 *@see        http://vietpad.sourceforge.net
 */
class FontDialog extends JDialog
{
    private final OpenList m_lstFontName;
    private final OpenList m_lstFontStyle;
    private final OpenList m_lstFontSize;
    private final JLabel m_preview;
    private final ResourceBundle myResources = ResourceBundle.getBundle("Resources");
    private Font curFont;
    private boolean m_succeeded = false;
    
    protected JCheckBox m_chkStrikethrough;
    protected JCheckBox m_chkSubscript;
    protected JCheckBox m_chkSuperscript;
    
    protected JComboBox m_cbColor;
    
    
    
    
    
    private MutableAttributeSet m_attributes;
    
    /**
     *  Constructor for the FontDialog object
     *
     *@param  owner  the Frame from which the dialog is displayed
     */
    public FontDialog(JFrame owner) {
        super(owner, ResourceBundle.getBundle("Resources").getString("Font"), true);
        setLocale(owner.getLocale());
        setResizable(false);
        final JPanel pp = new JPanel();
        pp.setBorder(new EmptyBorder(5, 10, 11, 9));
        pp.setLayout(new BoxLayout(pp, BoxLayout.Y_AXIS));
        JPanel p = new JPanel();
        p.setBorder(new TitledBorder(new EtchedBorder(), myResources.getString("Font")));
        
        final Font[] allFonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts();
        final Collection banglaFamilies = new TreeSet();
        for (int i = 0; i < allFonts.length; i++) {
            BPadLogger.logger.info("Checking if the font can display Bangla or English scripts");
            if (allFonts[i].canDisplay('\u0981') ||allFonts[i].canDisplay('\u0065')) {
                banglaFamilies.add(allFonts[i].getFamily());
            }
        }
        
        if (BanglaPad.MAC_OS_X) {
            final String[] abcFamilies = new String[]{
                    "Time", "Arial", "Courier New"
            };
            for (int i = 0; i < abcFamilies.length; i++) {
                if (new Font(".Vn" + abcFamilies[i], Font.PLAIN, 1).getPSName().startsWith("Vn")) {
                    banglaFamilies.add(".Vn" + abcFamilies[i]);
                    if (new Font(".Vn" + abcFamilies[i] + "H", Font.PLAIN, 1).getPSName().startsWith("Vn")) {
                        banglaFamilies.add(".Vn" + abcFamilies[i] + "H");
                    }
                }
            }
        }
        BPadLogger.logger.debug("Adding font names to font dialog");
        m_lstFontName = new OpenList(banglaFamilies.toArray(), myResources.getString("Name:"));
        p.add(m_lstFontName);
        
        BPadLogger.logger.debug("Adding font styles to the font dialog");
        m_lstFontStyle = new OpenList(
                new String[]{"Regular", "Bold", "Italic", "Bold Italic"},
                myResources.getString("Style:"),
                11);
        p.add(m_lstFontStyle);
        
        BPadLogger.logger.debug("Adding font sizes to the font dialog");
        m_lstFontSize = new OpenList(
                new String[]{"9", "10", "11", "12", "13", "14", "18", "24", "36", "48", "64", "72", "96"},
                myResources.getString("Size:"),
                5);
        p.add(m_lstFontSize);
        
        pp.add(p);
        
        m_lstFontName.setPreferredSizeList(m_lstFontStyle.getPreferredSizeList());
        // work around Bug ID: 4682565
        
        ListSelectionListener lsel =
            new ListSelectionListener()
            {
            public void valueChanged(ListSelectionEvent e) {
                SwingUtilities.invokeLater(
                        new Runnable()
                        {
                            public void run() {
                                BPadLogger.logger.debug("Calling updatePreview() as action of font name,size or styles changes");
                                updatePreview();
                            }
                        });
            }
            };
            m_lstFontName.addListSelectionListener(lsel);
            m_lstFontStyle.addListSelectionListener(lsel);
            m_lstFontSize.addListSelectionListener(lsel);
            
            m_lstFontSize.addActionListener(
                    new ActionListener()
                    {
                        public void actionPerformed(ActionEvent e) {
                            SwingUtilities.invokeLater(
                                    new Runnable()
                                    {
                                        public void run() {
                                            updatePreview();
                                        }
                                    });
                        }
                    });
            
            
            
            p = new JPanel(new GridLayout(2, 3, 10, 5));
            p.setBorder(new TitledBorder(new EtchedBorder(), "Effects"));
            m_chkStrikethrough = new JCheckBox("Strikeout");
            p.add(m_chkStrikethrough);
            m_chkSubscript = new JCheckBox("Subscript");
            p.add(m_chkSubscript);
            m_chkSuperscript = new JCheckBox("Superscript");
            p.add(m_chkSuperscript);
            pp.add(p);
            
            getContentPane().add(Box.createVerticalStrut(5));
            p = new JPanel();
            p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
            p.add(Box.createHorizontalStrut(10));
            p.add(new JLabel("Color:"));
            p.add(Box.createHorizontalStrut(5));
            m_cbColor = new JComboBox();
            int[] values = new int[] { 0, 128, 192, 255 };
            for (int r=0; r<values.length; r++) {
                for (int g=0; g<values.length; g++) {
                    for (int b=0; b<values.length; b++) {
                        Color c = new Color(values[r], values[g], values[b]);
                        m_cbColor.addItem(c);
                    }
                }
            }
            
            m_cbColor.setRenderer(new ColorComboRenderer());
            p.add(m_cbColor);
            p.add(Box.createHorizontalStrut(10));
            pp.add(p);
            
            ActionListener lst = new ActionListener() { 
                public void actionPerformed(ActionEvent e) {
                    updatePreview();
                }
            };
            m_cbColor.addActionListener(lst);
            
            
            
            p = new JPanel(new BorderLayout());
            p.setBorder(new TitledBorder(new EtchedBorder(), myResources.getString("Preview")));
            
            BPadLogger.logger.info("Adding different font preview");
            m_preview = new JLabel(
                    "\u0986\u09AA\u09A8\u09BF \u0995\u09C7\u09AE\u09A8 \u0986\u099B\u09C7\u09A8?",
                    JLabel.CENTER);
            m_preview.setBackground(Color.white);
            m_preview.setForeground(Color.black);
            m_preview.setOpaque(true);
            m_preview.setBorder(new LineBorder(Color.black));
            m_preview.setPreferredSize(new Dimension(120, 70));
            p.add(m_preview);
            pp.add(p);
            pp.add(Box.createVerticalStrut(9));
            
            p = new JPanel(null);
            p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
            p.add(Box.createHorizontalGlue());
            
            JButton btOK = new JButton(myResources.getString("OK"));
            btOK.addActionListener(
                    new ActionListener()
                    {
                        public void actionPerformed(ActionEvent e) {
                            m_succeeded = true;
                            dispose();
                        }
                    });
            
            JButton btCancel = new JButton(myResources.getString("Cancel"));
            btCancel.addActionListener(
                    new ActionListener()
                    {
                        public void actionPerformed(ActionEvent e) {
                            dispose();
                        }
                    });
            
            if (BanglaPad.MAC_OS_X) {
                p.add(btCancel);
                p.add(Box.createHorizontalStrut(7));
                p.add(btOK);
            } else {
                p.add(btOK);
                p.add(Box.createHorizontalStrut(5));
                p.add(btCancel);
                p.add(Box.createHorizontalStrut(2));
            }
            pp.add(p);
            
            getContentPane().add(pp);
            rootPane.setDefaultButton(btOK);
            pack();
            setLocationRelativeTo(owner);
            
            // Handle Escape key to hide the dialog
            KeyStroke escapeKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
            Action escapeAction =
                new AbstractAction()
                {
                public void actionPerformed(ActionEvent e) {
                    dispose();
                }
                };
                getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(escapeKeyStroke, "ESCAPE");
                getRootPane().getActionMap().put("ESCAPE", escapeAction);
    }
    
    
    /**
     *  Sets the attributes attribute of the FontDialog object
     *
     *@param  font  The new attributes value
     */
    
    public void setAttributes(AttributeSet a) {
        BPadLogger.logger.debug("Begining of setAttributes() of FontDialog class");
        m_attributes = new SimpleAttributeSet(a);
        String name = StyleConstants.getFontFamily(a);
        m_lstFontName.setSelected(name);
        int size = StyleConstants.getFontSize(a);
        m_lstFontSize.setSelectedInt(size);
        if (StyleConstants.isBold(a) && StyleConstants.isItalic(a)) {
            m_lstFontStyle.setSelected("Bold Italic");
        }
        else if (StyleConstants.isBold(a)) {
            m_lstFontStyle.setSelected("Bold");
        }
        else if (StyleConstants.isItalic(a)) {
            m_lstFontStyle.setSelected("Italic");
        }
        else {
            m_lstFontStyle.setSelected("Regular");
        }       
        m_chkStrikethrough.setSelected(StyleConstants.isStrikeThrough(a));
        m_chkSubscript.setSelected(StyleConstants.isSubscript(a));
        m_chkSuperscript.setSelected(StyleConstants.isSuperscript(a));
        m_cbColor.setSelectedItem(StyleConstants.getForeground(a));
        BPadLogger.logger.debug("End of setAttributes() of FontDialog class");
        BPadLogger.logger.debug("Calling updatePreview() of FontDialog class from setAttributes()");
        updatePreview();
    }
    
    public AttributeSet getAttributes() {
        BPadLogger.logger.debug("Begining of getAttributes() of FontDialog class");
        if (m_attributes == null)
            return null;
        StyleConstants.setFontFamily(m_attributes,
                m_lstFontName.getSelected());
        StyleConstants.setFontSize(m_attributes,
                m_lstFontSize.getSelectedInt());
        String fontStyle = m_lstFontStyle.getSelected();
        if (fontStyle.compareTo("Bold Italic") == 0) {
            StyleConstants.setBold(m_attributes,true);
            StyleConstants.setItalic(m_attributes,true);
        }
        else if (fontStyle.compareTo("Bold") == 0) {
            StyleConstants.setBold(m_attributes,true);
            StyleConstants.setItalic(m_attributes,false);
        }
        else if (fontStyle.compareTo("Italic") == 0) {
            StyleConstants.setItalic(m_attributes,true);
            StyleConstants.setBold(m_attributes,false);
        }
        else {
            StyleConstants.setItalic(m_attributes,false);
            StyleConstants.setBold(m_attributes,false);
        }
        StyleConstants.setStrikeThrough(m_attributes,
                m_chkStrikethrough.isSelected());
        StyleConstants.setSubscript(m_attributes,
                m_chkSubscript.isSelected());
        StyleConstants.setSuperscript(m_attributes,
                m_chkSuperscript.isSelected());
        StyleConstants.setForeground(m_attributes,
                (Color)m_cbColor.getSelectedItem());
        BPadLogger.logger.debug("Returning attributes from getAttributes() of FontDialog class");
        return m_attributes;
    }
    
    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public boolean succeeded() {
        return m_succeeded;
    }
    
    
    /**
     *  Gets the font attribute of the FontDialog object
     *
     *@return    The font value
     */
    public Font getFont() {
        return curFont;
    }
    
    
    /**
     *  Updates Font Preview
     */
    protected void updatePreview() {
        BPadLogger.logger.debug("Begining of updatePreview() of FontDialog class");
        String name = m_lstFontName.getSelected();
        int size = m_lstFontSize.getSelectedInt();
        if (size <= 0) {
            return;
        }
        int style = m_lstFontStyle.getSelectedIndex();
        Font fn = new Font(name, style, size);
        /**
         * FIXME: If Bangla font is selected then preview Bangla text  
         * Else If English font is selected then preview English text  
         */
        
        Color c = (Color)m_cbColor.getSelectedItem();
        m_preview.setForeground(c);
        
        if (fn.canDisplay('\u0065')){
            m_preview.setText("How are you?");
        }
        if(fn.canDisplay('\u0986')){
            m_preview.setText("\u0986\u09AA\u09A8\u09BF \u0995\u09C7\u09AE\u09A8 \u0986\u099B\u09C7\u09A8?");
        }
        m_preview.setFont(fn);
        m_preview.repaint();
        curFont = fn;
        BPadLogger.logger.debug("End of updatePreview() of FontDialog class");
    }
}
