package net.sf.banglapad;

import java.awt.*;
import java.awt.event.*;
import java.util.Locale;
import java.util.ResourceBundle;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

/**
 *  Preferences dialog
 *
 *@author     Quan Nguyen
 *@created    on June 10, 2003, 2:44 PM
 *@version    1.1, 18 October 2003
 */

public class PreferencesDialog extends JDialog
{
    // Variables declaration
    private ButtonGroup buttonGroup, buttonGroup2;
    private JButton jButtonOK;
    private JCheckBox jCheckBoxEOL;
    private JPanel jPanelGeneral, jPanel2, jPanel4, jPanelUI, jPanel6;
  
    private ResourceBundle myResources = ResourceBundle.getBundle("Resources");
    private JTabbedPane jTabbedPane;
    private JRadioButton jRButtonEng, jRButtonBangla;
    private boolean previousLocaleVN;


    /**
     *  Creates new form PreferencesDialog
     *
     *@param  owner  Owner of Preferences dialog
     *@param  modal  modal or non-modal
     */
    public PreferencesDialog(Frame owner, boolean modal) {
        super(owner, modal);
        setLocale(owner.getLocale());
        initComponents();

        //  Handle escape key to hide the dialog
        KeyStroke escapeKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
        Action escapeAction =
            new AbstractAction()
            {
                public void actionPerformed(ActionEvent e) {
                    BPadLogger.logger.debug("Escape is pressed to cancel the preference dialog");
                    setVisible(false);
                }
            };
        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(escapeKeyStroke, "ESCAPE");
        getRootPane().getActionMap().put("ESCAPE", escapeAction);
    }


    /**
     *  Initializes the form
     */
    private void initComponents() {
        final GridBagConstraints gridBagConstraints = new GridBagConstraints();
        jTabbedPane = new JTabbedPane();

        buttonGroup = new ButtonGroup();
        jPanelGeneral = new JPanel();
        jPanelUI = new JPanel();
        jPanel4 = new JPanel();
        jCheckBoxEOL = new JCheckBox();
        buttonGroup2 = new ButtonGroup();
        jPanel6 = new JPanel();
        jRButtonEng = new JRadioButton();
        jRButtonBangla = new JRadioButton();
        jButtonOK = new JButton();

        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setTitle(myResources.getString("Preferences"));
        setResizable(false);
        addWindowListener(
            new WindowAdapter()
            {
                public void windowClosing(WindowEvent evt) {
                    BPadLogger.logger.debug("close button have pressed ");
                    setVisible(false);
                    dispose();
                }
            });

        
        jPanel4.setLayout(new BoxLayout(jPanel4, BoxLayout.Y_AXIS));

        jPanel4.setBorder(new TitledBorder(myResources.getString("Line_Separators") + " (CRLF, LF)"));
        jCheckBoxEOL.setText(myResources.getString("Always_Save_with_Platform_Default"));
        jPanel4.add(jCheckBoxEOL);

        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        jPanelGeneral.add(jPanel4, gridBagConstraints);
        setOpaque(jPanelGeneral, false);
        jTabbedPane.addTab(myResources.getString("General"), jPanelGeneral);

        jPanelUI.setLayout(new GridLayout(0, 1));

        jPanel6.setLayout(new BoxLayout(jPanel6, BoxLayout.Y_AXIS));
        jPanel6.setBorder(new TitledBorder(myResources.getString("Language")));
        jRButtonEng.setText("System Default (" + BanglaPad.systemDefault.getDisplayLanguage() + ")");
        buttonGroup2.add(jRButtonEng);
        jPanel6.add(jRButtonEng);
        jRButtonBangla.setText(BanglaPad.BANGLA.getDisplayLanguage());
//      jRButtonBangla.setEnabled(!BanglaPad.MAC_OS_X);
        buttonGroup2.add(jRButtonBangla);
        jPanel6.add(jRButtonBangla);
        jPanelUI.add(jPanel6);
        jPanelUI.add(Box.createVerticalGlue());
        setOpaque(jPanelUI, false);
        jTabbedPane.addTab(myResources.getString("User_Interface"), jPanelUI);

        getContentPane().add(jTabbedPane, BorderLayout.CENTER);

        jButtonOK.setText(myResources.getString("OK"));
        getRootPane().setDefaultButton(jButtonOK);
        jButtonOK.addActionListener(
            new ActionListener()
            {
                public void actionPerformed(ActionEvent evt) {
                    BPadLogger.logger.debug("OK button have pressed");
                    setVisible(false);
                    if (previousLocaleVN != isLocaleVN()) {
                        BPadLogger.logger.debug("User chose to chage the language");
                        JOptionPane.showMessageDialog(PreferencesDialog.this.getOwner(),
                                myResources.getString("Restart_me"),
                                BPadLogger.APP_NAME, JOptionPane.INFORMATION_MESSAGE);
                    }
                    BPadLogger.logger.debug("Before calling retriveOption()  of BanglaPadWithTools");
                    ((BanglaPadWithTools) getOwner()).retrieveOptions();
                }
            });

        jPanel2 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        jPanel2.setBorder(new EmptyBorder(5, 5, 5, 5));
        jPanel2.add(jButtonOK);
        getContentPane().add(jPanel2, BorderLayout.SOUTH);

        pack();
        setLocationRelativeTo(getOwner());
    }


    /**
     *  Sets the platform default EOL
     *
     *@param  defaultEOLOn
     */
    void setDefaultEOL(boolean defaultEOLOn) {
        jCheckBoxEOL.setSelected(defaultEOLOn);
    }


    /**
     *  Gets the platform default EOL
     *
     *@return    boolean is platform default EOL selected?
     */
    boolean isDefaultEOL() {
        return jCheckBoxEOL.isSelected();
    }

    /**
     *  Sets the default locale
     *
     *@param  localeVN  The new localeVN value
     */
    void setLocaleVN(boolean localeVN) {
        if (localeVN) {
            jRButtonBangla.setSelected(true);
        } else {
            jRButtonEng.setSelected(true);
        }
        previousLocaleVN = localeVN;
    }


    /**
     *  Gets the default locale (this may not be the same as system's default locale)
     *
     *@return   localeVN    The new localeVN value
     */
    boolean isLocaleVN() {
        return jRButtonBangla.isSelected();
    }


    /**
     *  Sets the opaque attribute of a component and subcomponents. In the Mac OS X Aqua and Windows XP
     *  look-and-feel, a JPanel in a tabbed pane should not obscure the color of the background.
     *
     *@param    component   component
     *@param    isOpaque    true if this component and subcomponents should be opaque
     */
    public static void setOpaque(JComponent component, boolean isOpaque) {
        component.setOpaque(isOpaque);
        Component[] c = component.getComponents();
        for (int i = 0; i < c.length; i++) {
            if (c[i] instanceof JComponent    // in case of heavyweights
                    && !(c[i] instanceof JTextComponent || c[i] instanceof JList)) {
                setOpaque((JComponent) c[i], isOpaque);
            }
        }
    }

}
