package net.sf.banglapad;

/*
 * I am subclassing JTextPane so that I can overwrite processKeyEvent. The 
 * main reason is that I want to implement a dead key (The user types two 
 * keys, eg. ^c, and a single accented letter appears). I previously tried 
 * to do this with a key map. I was able to cut out the dead key character 
 * and put in an accented letter, but the second letter always followed 
 * it (^ee). I tried to consume the second key event without success. 
 * Cleve 1999/01/12
 * I've changed the BasicTextUI, so that I can implement an Esperanto 
 * Spell-Checker.
 * Cleve 1999/02/10
 * I'm adding a timer to repaint the screen every 3 seconds, because Swing
 * doesn't always repaint correctly.
 * Cleve 1999/02/15
 * Correct linesInDocument(). Base calculation on size of text pane.
 * Cleve 1999/07/26
 * Add support for Persian (reverse text) and key maps.
 * Cleve 2000/12/05
 * Spell Check English. Expanded keymap functionality.
 * Cleve 2002/10/10
 * New English spellchecker
 * Cleve 2004/08/19
 * Don't reverse numbers for reversed text.
 * Cleve 2004/11/29
 *
 * Mi subklasigas JTextPane por superskribi la metodon processKeyEvent. La 
 * cxefa kialo estas ke mi volas krei senpasxan klavon. (La uzanto tajpas du 
 * klavojn, ekzemple ^c, kaj unu supersignita litero aperas.) Antauxe mi 
 * provis fari cxi tion per klavmapo. Mi sukcesis eltondi la senpasxan signon 
 * kaj enmeti supersignitan literon, sed la dua litero cxiam sekvis gxin (^cc). 
 * Mi provis "mangxi" la duan klaveventon sen sukceso.
 * Klivo 1999/01/12
 * Mi sxangxis la tekstan interfacon (BasicTextUI), por enmeti Esperantan
 * kontrolilon de literumado.
 * Klivo 1999/02/10
 * Mi enmetas horlogxon por repentri la ekranon cxiun trian sekundon, cxar "Sving"
 * ne cxiam repentras gxuste.
 * Korektu linesInDocument(). Kalkulu laux la grandeco de la tekstujo.
 * Klivo 1999/07/26
 * Subtenu Persan lingvon (inversigu tekston) kaj klavmapojn.
 * Klivo 2000/12/05
 * Kontrolu literumadon de la angla. Fortigu klavmapan funkcion.
 * Klivo 2002/10/10
 * Nova literumilo de anglolingvo
 * Klivo 2004/08/19
 * Ne inversigu numerojn dum inversigo de teksto.
 * Klivo 2004/11/29
 */

import java.awt.*;
import java.awt.im.*;
import java.awt.event.*;
import java.awt.print.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class SimTextPane extends JTextPane implements Printable, ActionListener {

   private String sim_dir;  // working directory / labordosierujo

   private String file_separator = "\\";  // \ for Windows, / for Linux
                                          // \ por Vindozo, / por Linukso
   private boolean   first_action = true;
   public SimTextPane () {
      super();
      startup();
   }

   public SimTextPane (String sim_dir) {
      super();
      this.sim_dir = sim_dir;
      startup();
   }

   private void startup() {

      file_separator = System.getProperty("file.separator");
      setUI(new SimTextUI());
      fontSize16();
   }


   /*
    * fontSize16 - Set the font size to 16.
    * fontSize16 - Sxangxu la tiparan grandecon al 16.
    */
    private void fontSize16 () {
       Font font1 = getFont();
       setFont(font1.deriveFont(Font.PLAIN,16.0f));
    }


   public void actionPerformed(ActionEvent e) {

      // Must be timer. Devas esti horlogxo.
      if (first_action) {
         requestFocus();
         first_action = false;
      }
      repaint();
   }


   public int print (Graphics g, PageFormat pf, int index) {
      // Remove caret while printing.
      // Forigu tekstomontrilon dum presado.
      Caret caret = getCaret();
      caret.setVisible(false);
      // Disable Spellcheck temporarily.
      // Malaktivigu Kontroladon portempe.

      double left_x = pf.getImageableX();
      double top_y  = pf.getImageableY() - (pf.getImageableHeight() * index) - 3.0;
      // Why -3? The pages were not dividing properly. -3 was decided by experiment.
      // Kial -3? La pagxoj ne dividigxis gxuste. -3 estis fiksita per eksperimento.
      g.translate((int)left_x, (int)top_y);
      printAll(g);
      caret.setVisible(true);
      return(Printable.PAGE_EXISTS);
   }  // print

   /* 
    * linesInDocument - Calculate the number of lines in the document.
    * linesInDocument - Kalkulu la nombron da linioj en la dokumento.
    */
   public int linesInDocument() {
      //return getDocument().getDefaultRootElement().getElementCount();
      // Korektita 1999/07/27
      Dimension dmn = new Dimension();
      dmn = getSize(dmn);
      Double fh;
      fh = new Double(heightOfFont());
      return (dmn.height / fh.intValue());
   }


   /* 
    * heightOfFont - Calculate the height of the current font from FontMetrics.
    * heightOfFont - Kalkulu la altecon de la nuna tiparo per FontMetrics.
    */
   public double heightOfFont() {
      // Getting the font size from the font object does not work.
      // For reasons which I don't understand, a font object's point size
      // does not reflect the actual size it will have on the screen
      // or page. The correct size must be acquired from FontMetrics.
      // Tipara grandeco akirita de la tipara objekto estas malgxusta.
      // Pro kialoj kiujn mi ne komprenas, la "punkto"-grandeco de tiparo
      // ne spegulas la efektivan grandecon kiun gxi havos sur ekrano aux
      // pagxo. La gxusta grandeco devas esti akirita de FontMetrics.
      // font_size = text_pane.getFont().getSize();
      return getGraphics().getFontMetrics().getHeight();
   }

   /* 
    * linesInPage - Calculate the number of lines in a page.
    * linesInPage - Kalkulu la nombron da linioj en pagxo.
    */
   public int linesInPage(PageFormat pf) {
      return (int)(pf.getImageableHeight() / heightOfFont());
   }

   /* 
    * numberOfPages - Calculate the number of pages in the current document.
    * numberOfPages - Kalkulu la nombron da pagxoj en la nuna dokumento.
    */
   public int numberOfPages(PageFormat pf) {
      int lines_in_doc  = linesInDocument();
      int lines_in_page = linesInPage(pf);
      int number_of_pages = (lines_in_doc / lines_in_page);
      if ((lines_in_doc % lines_in_page) > 0) number_of_pages++;
      return number_of_pages;
   }  // numberOfPages


   // SimTextUI - I've extended the user interface (view) in order to
   // overwrite the update method. Check_spelling needs to draw a red 
   // underline under misspelled words.
   // Mi etendis la uz-interfacon por superskribi la "update"-metodon. 
   // Check_spelling (kontrolu literumadon) devas desegni rugxan linion 
   // sub misliterumitaj vortoj.  - Klivo
   public class SimTextUI extends BasicTextPaneUI {
      public SimTextUI() {
         super();  
      }
/*      public void update (Graphics g, JComponent c) {
         paint(g,c);
      }*/
   } // SimTextUI



}  // SimTextPane


