/**
 * This class has been taken from SimRedo
 */
package net.sf.banglapad;

/**
 * UserKeyMap.java
 *
 *@author     Naushad UzZaman (naushadzaman@gmail.com)
 *@created    September 20, 2005
 *@version    0.1, December 10, 2005
 *
 */
import java.awt.*;
import java.io.*;
import java.util.*;

public class UserKeyMap {
    
    private static boolean enabled;
    
    public static String[] key_table;   // No control keys. Sen regklavoj.
    public static String[] ctrl_table;  // control  - provizore ne uzu.
    public static String[] alt_table;   // alternative
    
    
    // Chars are two bytes long. Integers are 4. To speed
    // up searching, two characters will be stored as a
    // single integer in the keypairs table.
    
    private final static int MAX_KEYPAIRS = 500;
    public  static int number_of_keypairs;
    private static int keypair_index;
    
    public static int[]     keypairs;
    public static String[]  keypair_translation;
    
    
    public final static int table_size = ('~' - '!') + 1;
    
    public UserKeyMap () {
        key_table   = new String[table_size]; 
        ctrl_table  = new String[table_size];
        alt_table   = new String[table_size];
        keypairs    = new int[MAX_KEYPAIRS];
        keypair_translation = new String[MAX_KEYPAIRS];
        number_of_keypairs = 0;
        keypair_index = 0;
        enabled = false;
    }
    
    
    public static boolean isEnabled() {
        return enabled;
    }
    
    public static void setEnabled(boolean b) {
        if (b == true) enabled = true;
        else enabled = false;
    }
    
    
    public static void loadUserKeyMap(File keymap_file) {
        BPadLogger.logger.debug("Begining of loadUserKeyMap() of UserKeyMap class");
        // Clear out the translation tables.
        key_table   = new String[table_size]; 
        ctrl_table  = new String[table_size];
        alt_table   = new String[table_size];
        keypairs    = new int[MAX_KEYPAIRS];
        keypair_translation = new String[MAX_KEYPAIRS];
        number_of_keypairs = 0;
        keypair_index = 0;
        
        // If the keymap file was null, just exit.
        enabled = false;
        if (keymap_file == null) return;
        
        int   table_index;
        
        String   the_line;      
        char     the_char;      // key to convert  
        char     second_char;   // for deadkeys  
        String   rest_of_line;  
        
        if (keymap_file != null) {
            try {
                FileInputStream   fis   = new FileInputStream(keymap_file);
                InputStreamReader isr   = new InputStreamReader(fis,"UnicodeBig");
                BufferedReader    in    = new BufferedReader(isr);
                while (in.ready()) {
                    the_line = in.readLine();
                    if (the_line.length() > 1 && the_line.charAt(0) != ' ') {
                        rest_of_line = "";
                        if (the_line.toLowerCase().startsWith("alt")) {
                            if (the_line.length() >= 5) {
                                the_char = the_line.charAt(3);
                                rest_of_line = the_line.substring(4);
                                if (the_char >= '!' && the_char <= '~') {
                                    table_index = (int)(the_char - '!');
                                    alt_table[table_index] = convert_slash_u(rest_of_line);
                                }
                            }
                        } else  
                            if (the_line.toLowerCase().startsWith("2keys")) {
                                the_line = convert_slash_u(the_line);
                                if (the_line.length() >= 8 && 
                                        keypair_index < MAX_KEYPAIRS) {
                                    the_char = the_line.charAt(5);
                                    second_char = the_line.charAt(6);
                                    rest_of_line = the_line.substring(7);
                                    keypairs[keypair_index] =
                                        (int)((the_char * 0x10000) + second_char);
                                    keypair_translation[keypair_index] = rest_of_line;
                                    keypair_index++;
                                }
                            }
                            else {
                                the_char = the_line.charAt(0);
                                rest_of_line = the_line.substring(1);
                                if (the_char >= '!' && the_char <= '~' && 
                                        rest_of_line.length() > 0) {
                                    table_index = (int)(the_char - '!');
                                    key_table[table_index] = convert_slash_u(rest_of_line);
                                }
                            }
                    }
                }  // while
                
            } catch (IOException ioe) {
                BPadLogger.logger.error("Exception in loadKeyMap() of UserkeyMap:couldn't read keymap: "+ioe);
            }
            number_of_keypairs = keypair_index;
            enabled = true;
        }
        BPadLogger.logger.debug("End of loadUserKeyMap() of UserKeyMap class");
    } // end of "loadUserKeyMap"
    
    
    // Translate dead key. Traduku senpasxajn klavojn.
    public static String translate(char prev_key, char the_key) {
        
        //System.out.println(">> " + prev_key + " " + the_key);
        
        int index;
        int key_int = (int)((prev_key * 0x10000) + the_key);
        
        for (index = 0; index < number_of_keypairs; index++) {
            if (keypairs[index] == key_int) {
                return keypair_translation[index];
            }
        }
        
        return null;
        
    }  // translate
    
    
    
    // Translations for single key presses. 
    // Tradukoj pro unuopaj klav-premoj.
    public static String translate(char the_key, boolean ctrl_down, 
            boolean alt_down) {
        String[] table;
        int index;
        //System.out.println(">> " + the_key + " " + ctrl_down);
        
        if (the_key < '!' || the_key > '~') return null;
        index = the_key - '!';
        
        if (alt_down) {
            return alt_table[index];
        }
        else {
            return key_table[index];
        }
        
    }  // translate
    
    
    
    // This method converts unicode escape sequences (\u06F0) in a 
    private static String convert_slash_u(String to_convert) {
        
        String  converting = to_convert;
        String  front, uni, end;
        int     the_index;
        
        the_index = converting.indexOf("\\u");
        
        while (the_index != -1 && (the_index + 6 <= converting.length())) {
            front = converting.substring(0,the_index);
            uni   = convert_uni_escape(converting.substring(the_index+2,the_index+6));
            if (the_index+6 < converting.length()) 
                end  = converting.substring(the_index+6);
            else end = "";
            converting = front + uni + end;
            the_index = converting.indexOf("\\u");
        }  // while
        
        the_index = converting.indexOf("\\U");
        
        while (the_index != -1 && (the_index + 6 <= converting.length())) {
            front = converting.substring(0,the_index);
            uni   = convert_uni_escape(converting.substring(the_index+2,the_index+6));
            if (the_index+6 < converting.length()) 
                end   = converting.substring(the_index+6);
            else end = "";
            converting = front + uni + end;
            the_index = converting.indexOf("\\U");
        }  // while
        return converting;
        
    }  // convert_slash_u
    
    
    
    private static String convert_uni_escape(String uni) {
        char[] the_unicode_char = new char[1];
        try {
            the_unicode_char[0] = (char)Integer.parseInt(uni,16);
        } catch (NumberFormatException nfe) {
            return uni;
        }
        return new String(the_unicode_char);
    }
    
}  // UserKeyMap




